@@ -7,6 +7,8 @@
 #ifndef _GPSD_COMPILER_H_
 #define _GPSD_COMPILER_H_
 
+#include "gpsd_config.h"	/* is STD_ATOMIC_H defined? */
+
 /* Macro for declaring function with printf-like arguments. */
 # if __GNUC__ >= 3 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
 #define PRINTF_FUNC(format_index, arg_index) \
@@ -22,4 +24,20 @@
 #define UNUSED
 #endif
 
+/* Needed because 4.x versions of GCC are really annoying */
+#define ignore_return(funcall) \
+    do { \
+        ssize_t locresult = (funcall); \
+        assert(locresult != -23); \
+    } while (0)
+
+static /*@unused@*/ inline void memory_barrier(void)
+{
+#ifndef S_SPLINT_S
+#ifdef STD_ATOMIC_H
+    atomic_thread_fence(memory_order_seq_cst);
+#endif /* STD_ATOMIC_H */
+#endif /* S_SPLINT_S */
+}
+
 #endif /* _GPSD_COMPILER_H_ */
