@@ -165,7 +165,7 @@
 
 #pragma pack(1)
 // This is the data format of the satellite data from the garmin USB
-typedef struct
+typedef struct __attribute__((__packed__))
 {
     uint8_t svid;
     uint16_t snr;		// 0 - 0xffff
@@ -179,7 +179,7 @@ typedef struct
 /* Garmin D800_Pvt_Datetype_Type */
 /* packet type:  GARMIN_PKTID_PVT_DATA   52 */
 /* This is the data format of the position data from the garmin USB */
-typedef struct
+typedef struct __attribute__((__packed__))
 {
     float alt;			/* altitude above WGS 84 (meters) */
     float epe;			/* estimated position error, 2 sigma (meters)  */
@@ -206,7 +206,7 @@ typedef struct
 				 * beginning of the current week */
 } cpo_pvt_data;
 
-typedef struct
+typedef struct __attribute__((__packed__))
 {
     uint32_t cycles;
     double pr;
@@ -219,7 +219,7 @@ typedef struct
 
 /* packet type:  GARMIN_PKTID_RMD_DATA   53 */
 /* seems identical to the packet id 0x29 from the Garmin GPS 25 */
-typedef struct
+typedef struct __attribute__((__packed__))
 {
     double rcvr_tow;
     int16_t rcvr_wn;
@@ -227,7 +227,7 @@ typedef struct
 } cpo_rcv_data;
 
 // This is the packet format to/from the Garmin USB
-typedef struct
+typedef struct __attribute__((__packed__))
 {
     uint8_t mPacketType;
     uint8_t mReserved1;
@@ -237,7 +237,7 @@ typedef struct
     uint32_t mDataSize;
     union
     {
-	int8_t chars[MAX_BUFFER_SIZE];
+	//int8_t chars[MAX_BUFFER_SIZE];
 	uint8_t uchars[MAX_BUFFER_SIZE];
 	cpo_pvt_data pvt;
 	cpo_sat_data sats;
@@ -283,8 +283,10 @@ static inline double radtodeg(double rad)
 static gps_mask_t PrintSERPacket(struct gps_device_t *session,
 				 unsigned char pkt_id, int pkt_len,
 				 unsigned char *buf);
+#if defined(HAVE_LIBUSB)
 static gps_mask_t PrintUSBPacket(struct gps_device_t *session,
 				 Packet_t * pkt);
+#endif /* HAVE_LIBUSB */
 
 gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 			  int pkt_len, unsigned char *buf)
@@ -304,17 +306,17 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
     cpo_pvt_data *pvt = NULL;
     cpo_rcv_data *rmd = NULL;
 
-    gpsd_report(session->context->debug, LOG_DATA,
+    gpsd_report(&session->context->errout, LOG_DATA,
 		"Garmin: PrintSERPacket(, %#02x, %#02x, )\n", pkt_id, pkt_len);
 
     session->cycle_end_reliable = true;
 
     switch (pkt_id) {
     case ACK:
-	gpsd_report(session->context->debug, LOG_PROG, "Garmin: ACK\n");
+	gpsd_report(&session->context->errout, LOG_PROG, "Garmin: ACK\n");
 	break;
     case NAK:
-	gpsd_report(session->context->debug, LOG_PROG, "Garmin: NAK\n");
+	gpsd_report(&session->context->errout, LOG_PROG, "Garmin: NAK\n");
 	break;
     case GARMIN_PKTID_L001_COMMAND_DATA:
 	prod_id = get_uint16((uint8_t *) buf);
@@ -339,11 +341,11 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	    break;
 	}
 	/*@ +branchstate @*/
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Appl, Command Data: %s\n", msg);
 	break;
     case GARMIN_PKTID_PRODUCT_RQST:
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Appl, Product Data req\n");
 	break;
     case GARMIN_PKTID_PRODUCT_DATA:
@@ -351,23 +353,23 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	ver = get_uint16((uint8_t *) & buf[2]);
 	maj_ver = (int)(ver / 100);
 	min_ver = (int)(ver - (maj_ver * 100));
-	gpsd_report(session->context->debug, LOG_PROG, 
+	gpsd_report(&session->context->errout, LOG_PROG, 
 		    "Garmin: Appl, Product Data, sz: %d\n",
 		    pkt_len);
 	(void)snprintf(session->subtype, sizeof(session->subtype),
 		       "%d: %d.%02d", (int)prod_id, maj_ver, min_ver);
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "Garmin: Product ID: %d, SoftVer: %d.%02d\n",
 		    prod_id, maj_ver, min_ver);
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "Garmin: Product Desc: %s\n", &buf[4]);
 	mask |= DEVICEID_SET;
-	gpsd_report(session->context->debug, LOG_DATA,
+	gpsd_report(&session->context->errout, LOG_DATA,
 		    "Garmin: PRODUCT_DATA: subtype=%s\n",
 		    session->subtype);
 	break;
     case GARMIN_PKTID_PVT_DATA:
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Appl, PVT Data Sz: %d\n", pkt_len);
 
 	pvt = (cpo_pvt_data *) buf;
@@ -382,29 +384,29 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	time_l += (time_t) round(pvt->gps_tow);
 	session->context->gps_tow = pvt->gps_tow;
 	session->newdata.time = (timestamp_t)time_l;
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: time_l: %ld\n", (long int)time_l);
 
 	session->newdata.latitude = radtodeg(pvt->lat);
 	/* sanity check the lat */
 	if (90.0 < session->newdata.latitude) {
 	    session->newdata.latitude = 90.0;
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin: ERROR: Latitude overrange\n");
 	} else if (-90.0 > session->newdata.latitude) {
 	    session->newdata.latitude = -90.0;
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin: ERROR: Latitude negative overrange\n");
 	}
 	session->newdata.longitude = radtodeg(pvt->lon);
 	/* sanity check the lon */
 	if (180.0 < session->newdata.longitude) {
 	    session->newdata.longitude = 180.0;
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin: ERROR: Longitude overrange\n");
 	} else if (-180.0 > session->newdata.longitude) {
 	    session->newdata.longitude = -180.0;
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin: ERROR: Longitude negative overrange\n");
 	}
 	// altitude over WGS84 converted to MSL
@@ -466,20 +468,20 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	    break;
 	}
 
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Appl, mode %d, status %d\n",
 		    session->newdata.mode, session->gpsdata.status);
 
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "Garmin: UTC Time: %lf\n",
 		    session->newdata.time);
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "Garmin: Geoid Separation (MSL-WGS84): from garmin %lf, calculated %lf\n",
 		    -pvt->msl_hght,
 		    wgs84_separation(session->newdata.latitude,
 				     session->newdata.longitude));
 
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "Garmin: Alt: %.3f, Epe: %.3f, Eph: %.3f, Epv: %.3f, Fix: %d, Gps_tow: %f, Lat: %.3f, Lon: %.3f, LonVel: %.3f, LatVel: %.3f, AltVel: %.3f, MslHgt: %.3f, Leap: %d, GarminDays: %d\n",
 		    pvt->alt, pvt->epe, pvt->eph, pvt->epv, GPSD_LE16TOH(pvt->fix),
 		    pvt->gps_tow, session->newdata.latitude,
@@ -501,7 +503,7 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	    if (session->fixcnt > 3)
 		mask |= PPSTIME_IS;
 	}
-	gpsd_report(session->context->debug, LOG_DATA,
+	gpsd_report(&session->context->errout, LOG_DATA,
 		    "Garmin: PVT_DATA: time=%.2f, lat=%.2f lon=%.2f "
 		    "speed=%.2f track=%.2f climb=%.2f "
 		    "epx=%.2f epy=%.2f epv=%.2f "
@@ -521,13 +523,13 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
     case GARMIN_PKTID_RMD_DATA:
     case GARMIN_PKTID_RMD41_DATA:
 	rmd = (cpo_rcv_data *) buf;
-	gpsd_report(session->context->debug, LOG_DATA,
+	gpsd_report(&session->context->errout, LOG_DATA,
 		    "Garmin: PVT RMD Data Sz: %d\n", pkt_len);
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: PVT RMD rcvr_tow: %f, rcvr_wn: %d\n",
 		    rmd->rcvr_tow, rmd->rcvr_wn);
 	for (i = 0; i < GARMIN_CHANNELS; i++) {
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin: PVT RMD Sat: %3u, cycles: %9u, pr: %16.6f, "
 			"phase: %7.3f, slp_dtct: %3s, snr: %3u, Valid: %3s\n",
 			(int)rmd->sv[i].svid + 1,
@@ -541,16 +543,15 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	break;
 
     case GARMIN_PKTID_SAT_DATA:
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: SAT Data Sz: %d\n", pkt_len);
 	sats = (cpo_sat_data *) buf;
 
 	session->gpsdata.satellites_visible = 0;
 	session->gpsdata.satellites_used = 0;
-	memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
 	gpsd_zero_satellites(&session->gpsdata);
 	for (i = 0, j = 0; i < GARMIN_CHANNELS; i++, sats++) {
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin:   Sat %3d, snr: %5u, elev: %2d, Azmth: %3d, Stat: %x\n",
 			sats->svid, GPSD_LE16TOH(sats->snr), sats->elev,
 			GPSD_LE16TOH(sats->azmth),
@@ -563,15 +564,15 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	    }
 
 	    if ((int)sats->svid <= 32)
-		session->gpsdata.PRN[j] = (int)sats->svid;	/* GPS */
+		session->gpsdata.skyview[j].PRN = (short)sats->svid;	/* GPS */
 	    else
-		session->gpsdata.PRN[j] = (int)sats->svid + 87;	/* SBAS */
-	    session->gpsdata.azimuth[j] = (int)GPSD_LE16TOH(sats->azmth);
-	    session->gpsdata.elevation[j] = (int)sats->elev;
+		session->gpsdata.skyview[j].PRN = (short)sats->svid + 87;	/* SBAS */
+	    session->gpsdata.skyview[j].azimuth = (short)GPSD_LE16TOH(sats->azmth);
+	    session->gpsdata.skyview[j].elevation = (short)sats->elev;
 	    // Garmin does not document this.  snr is in dB*100
 	    // Known, but not seen satellites have a dB value of -1*100
-	    session->gpsdata.ss[j] = (float)(GPSD_LE16TOH(sats->snr) / 100.0);
-	    if (session->gpsdata.ss[j] == -1) {
+	    session->gpsdata.skyview[j].ss = (float)(GPSD_LE16TOH(sats->snr) / 100.0);
+	    if (session->gpsdata.skyview[j].ss == -1) {
 		continue;
 	    }
 	    // FIX-ME: Garmin documents this, but Daniel Dorau
@@ -579,8 +580,8 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	    // doesn't match it.
 	    if ((uint8_t) 0 != (sats->status & 4)) {
 		// used in solution?
-		session->gpsdata.used[session->gpsdata.satellites_used++]
-		    = session->gpsdata.PRN[j];
+		session->gpsdata.skyview[j].used = true;
+		session->gpsdata.satellites_used++;
 	    }
 	    session->gpsdata.satellites_visible++;
 	    j++;
@@ -591,7 +592,7 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
 	    mask |= SATELLITE_SET;
 	if (session->gpsdata.satellites_used > 0)
 	    mask |= USED_IS;
-	gpsd_report(session->context->debug, LOG_DATA,
+	gpsd_report(&session->context->errout, LOG_DATA,
 		    "Garmin: SAT_DATA: visible=%d used=%d\n",
 		    session->gpsdata.satellites_visible,
 		    session->gpsdata.satellites_used);
@@ -599,28 +600,29 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id,
     case GARMIN_PKTID_PROTOCOL_ARRAY:
 	// this packet is never requested, it just comes, in some case
 	// after a GARMIN_PKTID_PRODUCT_RQST
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "Garmin: Appl, Product Capability, sz: %d\n",
 		    pkt_len);
 	for (i = 0; i < pkt_len; i += 3) {
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin:   %c%03d\n", 
 			buf[i],	get_uint16((uint8_t *) & buf[i + 1]));
 	}
 	break;
     default:
-	gpsd_report(session->context->debug, LOG_WARN,
+	gpsd_report(&session->context->errout, LOG_WARN,
 		    "Garmin: Unknown packet id: %#02x, Sz: %#02x\n",
 		    pkt_id, pkt_len);
 	break;
     }
-    gpsd_report(session->context->debug, LOG_DATA,
+    gpsd_report(&session->context->errout, LOG_DATA,
 		"Garmin: PrintSERPacket(, %#02x, %#02x, )\n",
 		pkt_id, pkt_len);
     return mask;
 }
 
 
+#if defined(HAVE_LIBUSB)
 /*@ -branchstate @*/
 // This works around cppcheck not looking into enough config branches
 // cppcheck-suppress unusedFunction
@@ -639,10 +641,10 @@ static gps_mask_t PrintUSBPacket(struct gps_device_t *session, Packet_t * pkt)
 //
     uint8_t *buffer = (uint8_t *) pkt;
 
-    gpsd_report(session->context->debug, LOG_PROG, "Garmin: PrintUSBPacket()\n");
+    gpsd_report(&session->context->errout, LOG_PROG, "Garmin: PrintUSBPacket()\n");
 // gem
     if (DLE == pkt->mPacketType) {
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: really a SER packet!\n");
 	return PrintSERPacket(session,
 			      (unsigned char)buffer[1],
@@ -650,30 +652,28 @@ static gps_mask_t PrintUSBPacket(struct gps_device_t *session, Packet_t * pkt)
     }
 // gem
     if (4096 < mDataSize) {
-	gpsd_report(session->context->debug, LOG_WARN,
+	gpsd_report(&session->context->errout, LOG_WARN,
 		    "Garmin: bogus packet, size too large=%d\n",
 		    mDataSize);
 	return 0;
     }
 
-    (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag), "%u",
-		   (unsigned int)pkt->mPacketType);
     switch (pkt->mPacketType) {
     case GARMIN_LAYERID_TRANSPORT:
 	/* Garmin USB layer specific */
 	switch (pkt->mPacketId) {
 	case GARMIN_PKTID_TRANSPORT_START_SESSION_REQ:
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: Transport, Start Session req\n");
 	    break;
 	case GARMIN_PKTID_TRANSPORT_START_SESSION_RESP:
 	    mode = get_int32(&pkt->mData.uchars[0]);
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: Transport, Start Session resp, unit: 0x%x\n",
 			mode);
 	    break;
 	default:
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: Transport, Packet: Type %d %d %d, ID: %d, Sz: %d\n",
 			pkt->mPacketType, pkt->mReserved1, pkt->mReserved2,
 			pkt->mPacketId, mDataSize);
@@ -693,11 +693,11 @@ static gps_mask_t PrintUSBPacket(struct gps_device_t *session, Packet_t * pkt)
 	switch (pkt->mPacketId) {
 	case PRIV_PKTID_SET_MODE:
 	    prod_id = get_uint16(&pkt->mData.uchars[0]);
-	    gpsd_report(session->context->debug, LOG_PROG, 
+	    gpsd_report(&session->context->errout, LOG_PROG, 
 			"Garmin: Private, Set Mode: %d\n", prod_id);
 	    break;
 	case PRIV_PKTID_INFO_REQ:
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: Private, ID: Info Req\n");
 	    break;
 	case PRIV_PKTID_INFO_RESP:
@@ -706,21 +706,21 @@ static gps_mask_t PrintUSBPacket(struct gps_device_t *session, Packet_t * pkt)
 	    min_ver = (int)(veri & 0xffff);
 	    mode = get_int32(&pkt->mData.uchars[4]);
 	    serial = get_int32(&pkt->mData.uchars[8]);
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: Private, ID: Info Resp\n");
-	    gpsd_report(session->context->debug, LOG_INF,
+	    gpsd_report(&session->context->errout, LOG_INF,
 			"Garmin: USB Driver found, Version %d.%d, Mode: %d, GPS Serial# %u\n",
 			maj_ver, min_ver, mode, serial);
 	    break;
 	default:
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: Private, Packet: ID: %d, Sz: %d\n",
 			pkt->mPacketId, mDataSize);
 	    break;
 	}
 	break;
     default:
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Packet: Type %d %d %d, ID: %d, Sz: %d\n",
 		    pkt->mPacketType, pkt->mReserved1, pkt->mReserved2,
 		    pkt->mPacketId, mDataSize);
@@ -731,9 +731,10 @@ static gps_mask_t PrintUSBPacket(struct gps_device_t *session, Packet_t * pkt)
 }
 
 /*@ +branchstate @*/
+#endif /* HAVE_LIBUSB */
 
 
-#if defined(__linux__) || defined(S_SPLINT_S)
+#if defined(HAVE_LIBUSB) && (defined(__linux__) || defined(S_SPLINT_S))
 /* build and send a packet w/ USB protocol */
 static void Build_Send_USB_Packet(struct gps_device_t *session,
 				  uint32_t layer_id, uint32_t pkt_id,
@@ -756,7 +757,7 @@ static void Build_Send_USB_Packet(struct gps_device_t *session,
 
     theBytesReturned = gpsd_write(session, (const char *)thePacket,
 				  (size_t) theBytesToWrite);
-    gpsd_report(session->context->debug, LOG_PROG,
+    gpsd_report(&session->context->errout, LOG_PROG,
 		"Garmin: SendPacket(), wrote %zd bytes\n",
 		theBytesReturned);
 
@@ -773,7 +774,7 @@ static void Build_Send_USB_Packet(struct gps_device_t *session,
 	(void)gpsd_write(session, n, 0);
     }
 }
-#endif /* __linux__ || S_SPLINT_S */
+#endif /* HAVE_LIBUSB && (__linux__ || S_SPLINT_S) */
 
 /* build and send a packet in serial protocol */
 /* layer_id unused */
@@ -829,7 +830,7 @@ static void Build_Send_SER_Packet(struct gps_device_t *session,
 
     theBytesReturned = gpsd_write(session, (const char *)thePacket,
 				  (size_t) theBytesToWrite);
-    gpsd_report(session->context->debug, LOG_PROG,
+    gpsd_report(&session->context->errout, LOG_PROG,
 		"Garmin: SendPacket(), wrote %zd bytes\n",
 		theBytesReturned);
 
@@ -849,7 +850,7 @@ static void Build_Send_SER_Packet(struct gps_device_t *session,
 // This works around cppcheck not looking into enough config branches
 // cppcheck-suppress unusedFunction
 static bool is_usb_device(const char *path UNUSED, int vendor, int product,
-			  const int debug)
+			  struct gpsd_errout_t *errout)
 {
     // discover devices
     libusb_device **list;
@@ -857,12 +858,12 @@ static bool is_usb_device(const char *path UNUSED, int vendor, int product,
     ssize_t i = 0;
     bool found = false;
 
-    gpsd_report(debug, LOG_INF, 
+    gpsd_report(errout, LOG_INF, 
 		"attempting USB device enumeration.\n");
     (void)libusb_init(NULL);
 
     if ((cnt = libusb_get_device_list(NULL, &list)) < 0) {
-	gpsd_report(debug, LOG_ERROR,
+	gpsd_report(errout, LOG_ERROR,
 		    "USB device list call failed.\n");
 	libusb_exit(NULL);
 	return false;
@@ -874,13 +875,13 @@ static bool is_usb_device(const char *path UNUSED, int vendor, int product,
 
 	int r = libusb_get_device_descriptor(dev, &desc);
 	if (r < 0) {
-	    gpsd_report(debug, LOG_ERROR,
+	    gpsd_report(errout, LOG_ERROR,
 			"USB descriptor fetch failed on device %zd.\n", i);
 	    continue;
 	}
 
 	/* we can extract device descriptor data */
-	gpsd_report(debug, LOG_INF,
+	gpsd_report(errout, LOG_INF,
 		    "%04x:%04x (bus %d, device %d)\n",
 		    desc.idVendor, desc.idProduct,
 		    libusb_get_bus_number(dev),
@@ -893,7 +894,7 @@ static bool is_usb_device(const char *path UNUSED, int vendor, int product,
 	}
     }
 
-    gpsd_report(debug, LOG_INF,
+    gpsd_report(errout, LOG_INF,
 		"vendor/product match with %04x:%04x %sfound\n",
 		vendor, product, found ? "" : "not ");
     libusb_free_device_list(list, 1);
@@ -943,11 +944,11 @@ static bool garmin_usb_detect(struct gps_device_t *session UNUSED)
     else {
 #ifdef HAVE_LIBUSB
 	if (!is_usb_device(session->gpsdata.dev.path, 0x091e, 0x0003,
-		session->context->debug))
+		&session->context->errout))
 	    return false;
 
 	if (!gpsd_set_raw(session)) {
-	    gpsd_report(session->context->debug, LOG_ERROR,
+	    gpsd_report(&session->context->errout, LOG_ERROR,
 			"Garmin: garmin_usb_detect: error changing port attributes: %s\n",
 			strerror(errno));
 	    return false;
@@ -955,7 +956,7 @@ static bool garmin_usb_detect(struct gps_device_t *session UNUSED)
 
 	if (sizeof(session->driver.garmin.Buffer) < sizeof(Packet_t)) {
 	    /* dunno how this happens, but it does on some compilers */
-	    gpsd_report(session->context->debug, LOG_ERROR,
+	    gpsd_report(&session->context->errout, LOG_ERROR,
 			"Garmin: garmin_usb_detect: Compile error, garmin.Buffer too small.\n");
 	    return false;
 	}
@@ -963,7 +964,7 @@ static bool garmin_usb_detect(struct gps_device_t *session UNUSED)
 	// FIXME!!! needs to use libusb totally and move garmin_gps aside */
 	// set Mode 1, mode 0 is broken somewhere past 2.6.14
 	// but how?
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Set garmin_gps driver mode = 0\n");
 	Build_Send_USB_Packet(session, GARMIN_LAYERID_PRIVATE,
 			      PRIV_PKTID_SET_MODE, 4, MODE_GARMIN_SERIAL);
@@ -989,12 +990,12 @@ static void garmin_event_hook(struct gps_device_t *session, event_t event)
      */
     if (event == event_identified || event == event_reactivate) {
 	// Tell the device to send product data
-	gpsd_report(session->context->debug, LOG_PROG, "Garmin: Get Product Data\n");
+	gpsd_report(&session->context->errout, LOG_PROG, "Garmin: Get Product Data\n");
 	Build_Send_SER_Packet(session, GARMIN_LAYERID_APPL,
 			      GARMIN_PKTID_PRODUCT_RQST, 0, 0);
 
 	// turn on PVT data 49
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: Set to send reports every 1 second\n");
 
 	Build_Send_SER_Packet(session, GARMIN_LAYERID_APPL,
@@ -1003,7 +1004,7 @@ static void garmin_event_hook(struct gps_device_t *session, event_t event)
 
 #if USE_RMD
 	// turn on RMD data 110
-	gpsd_report(session->context->debug, LOG_PROG, 
+	gpsd_report(&session->context->errout, LOG_PROG, 
 		    "Garmin: Set to send Raw sat data\n");
 	Build_Send_SER_Packet(session, GARMIN_LAYERID_APPL,
 			      GARMIN_PKTID_L001_COMMAND_DATA, 2,
@@ -1012,7 +1013,7 @@ static void garmin_event_hook(struct gps_device_t *session, event_t event)
     }
     if (event == event_deactivate)
 	/* FIX-ME: is any action needed, or is closing the port sufficient? */
-	gpsd_report(session->context->debug, LOG_PROG,
+	gpsd_report(&session->context->errout, LOG_PROG,
 		    "Garmin: garmin_close()\n");
 }
 
@@ -1022,8 +1023,8 @@ static void garmin_event_hook(struct gps_device_t *session, event_t event)
 /*@ +charint @*/
 gps_mask_t garmin_ser_parse(struct gps_device_t *session)
 {
-    unsigned char *buf = session->packet.outbuffer;
-    size_t len = session->packet.outbuflen;
+    unsigned char *buf = session->lexer.outbuffer;
+    size_t len = session->lexer.outbuflen;
     unsigned char data_buf[MAX_BUFFER_SIZE];
     unsigned char c;
     int i = 0;
@@ -1035,24 +1036,24 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     unsigned char chksum = 0;
     gps_mask_t mask = 0;
 
-    gpsd_report(session->context->debug, LOG_RAW, "Garmin: garmin_ser_parse()\n");
+    gpsd_report(&session->context->errout, LOG_RAW, "Garmin: garmin_ser_parse()\n");
     if (6 > len) {
 	/* WTF? */
 	/* minimum packet; <DLE> [pkt id] [length=0] [chksum] <DLE> <STX> */
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: serial too short: %zd\n", len);
 	return 0;
     }
     /* debug */
     for (i = 0; i < (int)len; i++) {
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: Char: %#02x\n", buf[i]);
     }
 
     if ('\x10' != buf[0]) {
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1, "Garmin: buf[0] not DLE\n");
+	gpsd_report(&session->context->errout, LOG_RAW + 1, "Garmin: buf[0] not DLE\n");
 	return 0;
     }
     n = 1;
@@ -1061,7 +1062,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     if ('\x10' == pkt_id) {
 	if ('\x10' != buf[n++]) {
 	    Send_NAK();
-	    gpsd_report(session->context->debug, LOG_RAW + 1, "Garmin: Bad pkt_id %#02x\n", pkt_id);
+	    gpsd_report(&session->context->errout, LOG_RAW + 1, "Garmin: Bad pkt_id %#02x\n", pkt_id);
 	    return 0;
 	}
     }
@@ -1070,7 +1071,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     chksum += pkt_len;
     if ('\x10' == pkt_len) {
 	if ('\x10' != buf[n++]) {
-	    gpsd_report(session->context->debug, LOG_RAW + 1, 
+	    gpsd_report(&session->context->errout, LOG_RAW + 1, 
 			"Garmin: Bad pkt_len %#02x\n", pkt_len);
 	    Send_NAK();
 	    return 0;
@@ -1084,7 +1085,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
 	    break;
 	}
 	if (len < n + i) {
-	    gpsd_report(session->context->debug, LOG_RAW + 1,
+	    gpsd_report(&session->context->errout, LOG_RAW + 1,
 			"Garmin: Packet too short %zd < %zd\n",
 			len, n + i);
 	    Send_NAK();
@@ -1095,7 +1096,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
 	    got_dle = 0;
 	    if ('\x10' != c) {
 		Send_NAK();
-		gpsd_report(session->context->debug, LOG_RAW + 1,
+		gpsd_report(&session->context->errout, LOG_RAW + 1,
 			    "Garmin: Bad DLE %#02x\n", c);
 		return 0;
 	    }
@@ -1110,7 +1111,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     /* get checksum */
     if (len < n + i) {
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: No checksum, Packet too short %zd < %zd\n", len,
 		    n + i);
 	return 0;
@@ -1120,7 +1121,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     /* get final DLE */
     if (len < n + i) {
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: No final DLE, Packet too short %zd < %zd\n", len,
 		    n + i);
 	return 0;
@@ -1128,13 +1129,13 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     c = buf[n + i++];
     if ('\x10' != c) {
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1, "Garmin: Final DLE not DLE\n");
+	gpsd_report(&session->context->errout, LOG_RAW + 1, "Garmin: Final DLE not DLE\n");
 	return 0;
     }
     /* get final ETX */
     if (len < n + i) {
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: No final ETX, Packet too short %zd < %zd\n", len,
 		    n + i);
 	return 0;
@@ -1143,7 +1144,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     c = buf[n + i];
     if ('\x03' != c) {
 	Send_NAK();
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: Final ETX not ETX\n");
 	return 0;
     }
@@ -1151,12 +1152,12 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     /* debug */
     /*@ -usedef -compdef @*/
     for (i = 0; i < data_index; i++) {
-	gpsd_report(session->context->debug, LOG_RAW + 1,
+	gpsd_report(&session->context->errout, LOG_RAW + 1,
 		    "Garmin: Char: %#02x\n", data_buf[i]);
     }
 
 
-    gpsd_report(session->context->debug, LOG_DATA,
+    gpsd_report(&session->context->errout, LOG_DATA,
 		"Garmin: garmin_ser_parse() Type: %#02x, Len: %#02x, chksum: %#02x\n",
 		pkt_id, pkt_len, chksum);
     mask = PrintSERPacket(session, pkt_id, pkt_len, data_buf);
@@ -1166,7 +1167,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     (void)usleep(300);
     Send_ACK();
     /*@ +usedef +compdef @*/
-    gpsd_report(session->context->debug, LOG_DATA, "Garmin: garmin_ser_parse( )\n");
+    gpsd_report(&session->context->errout, LOG_DATA, "Garmin: garmin_ser_parse( )\n");
     return mask;
 }
 
@@ -1194,11 +1195,11 @@ static void garmin_switcher(struct gps_device_t *session, int mode)
 	ssize_t status = gpsd_write(session, switcher, sizeof(switcher));
 	/*@ -charint @*/
 	if (status == (ssize_t)sizeof(switcher)) {
-	    gpsd_report(session->context->debug, LOG_PROG,
+	    gpsd_report(&session->context->errout, LOG_PROG,
 			"Garmin: => GPS: turn off binary %02x %02x %02x... \n",
 			switcher[0], switcher[1], switcher[2]);
 	} else {
-	    gpsd_report(session->context->debug, LOG_ERROR,
+	    gpsd_report(&session->context->errout, LOG_ERROR,
 			"Garmin: => GPS: FAILED\n");
 	}
 	settle();		// wait 333mS, essential!
@@ -1280,9 +1281,9 @@ static int GetPacket(struct gps_device_t *session)
     memset(session->driver.garmin.Buffer, 0, sizeof(Packet_t));
     memset(&delay, 0, sizeof(delay));
     session->driver.garmin.BufferLen = 0;
-    session->packet.outbuflen = 0;
+    session->lexer.outbuflen = 0;
 
-    gpsd_report(session->context->debug, LOG_DATA, "Garmin: GetPacket()\n");
+    gpsd_report(&session->context->errout, LOG_DATA, "Garmin: GetPacket()\n");
 
     for (cnt = 0; cnt < 10; cnt++) {
 	size_t pkt_size;
@@ -1303,18 +1304,18 @@ static int GetPacket(struct gps_device_t *session)
 	if (0 > theBytesReturned) {
 	    // read error...
 	    // or EAGAIN, but O_NONBLOCK is never set
-	    gpsd_report(session->context->debug, LOG_ERROR,
+	    gpsd_report(&session->context->errout, LOG_ERROR,
 			"Garmin: GetPacket() read error=%d, errno=%d\n",
 			theBytesReturned, errno);
 	    continue;
 	}
-	gpsd_report(session->context->debug, LOG_RAW,
+	gpsd_report(&session->context->errout, LOG_RAW,
 		    "Garmin: got %d bytes\n", theBytesReturned);
 
 	session->driver.garmin.BufferLen += theBytesReturned;
 	if (256 <= session->driver.garmin.BufferLen) {
 	    // really bad read error...
-	    gpsd_report(session->context->debug, LOG_ERROR,
+	    gpsd_report(&session->context->errout, LOG_ERROR,
 			"Garmin: GetPacket() packet too long, %ld > 255 !\n",
 			session->driver.garmin.BufferLen);
 	    session->driver.garmin.BufferLen = 0;
@@ -1325,7 +1326,7 @@ static int GetPacket(struct gps_device_t *session)
 	    // have enough data to check packet size
 	    if (session->driver.garmin.BufferLen > pkt_size) {
 		// wrong amount of data in buffer
-		gpsd_report(session->context->debug, LOG_ERROR,
+		gpsd_report(&session->context->errout, LOG_ERROR,
 			    "Garmin: GetPacket() packet size wrong! Packet: %ld, s/b %ld\n",
 			    session->driver.garmin.BufferLen, pkt_size);
 		session->driver.garmin.BufferLen = 0;
@@ -1347,24 +1348,24 @@ static int GetPacket(struct gps_device_t *session)
     }
     // dump the individual bytes, debug only
     // for ( x = 0; x < session->driver.garmin.BufferLen; x++ ) {
-    // gpsd_report(session->context->debug, LOG_RAW+1, "Garmin: p[%d] = %x\n", x, session->driver.garmin.Buffer[x]);
+    // gpsd_report(&session->context->errout, LOG_RAW+1, "Garmin: p[%d] = %x\n", x, session->driver.garmin.Buffer[x]);
     // }
     if (10 <= cnt) {
-	gpsd_report(session->context->debug, LOG_ERROR,
+	gpsd_report(&session->context->errout, LOG_ERROR,
 		    "Garmin: GetPacket() packet too long or too slow!\n");
 	return -1;
     }
 
-    gpsd_report(session->context->debug, LOG_RAW,
+    gpsd_report(&session->context->errout, LOG_RAW,
 		"Garmin: GotPacket() sz=%d \n",
 		session->driver.garmin.BufferLen);
-    session->packet.outbuflen = session->driver.garmin.BufferLen;
+    session->lexer.outbuflen = session->driver.garmin.BufferLen;
     return 0;
 }
 
 static gps_mask_t garmin_usb_parse(struct gps_device_t *session)
 {
-    gpsd_report(session->context->debug,  LOG_PROG, "Garmin: garmin_usb_parse()\n");
+    gpsd_report(&session->context->errout,  LOG_PROG, "Garmin: garmin_usb_parse()\n");
     return PrintUSBPacket(session,
 			  (Packet_t *) session->driver.garmin.Buffer);
 }
@@ -1386,6 +1387,7 @@ const struct gps_type_t driver_garmin_usb_binary_old =
     .get_packet     = garmin_get_packet,/* how to grab a packet */
     .parse_packet   = garmin_usb_parse,	/* parse message packets */
     .rtcm_writer    = NULL,		/* don't send DGPS corrections */
+    .init_query     = NULL,		/* non-perturbing initial query */
     .event_hook     = garmin_event_hook,/* lifetime event handler */
 #ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
@@ -1415,6 +1417,7 @@ const struct gps_type_t driver_garmin_usb_binary =
     .get_packet     = generic_get,      /* how to grab a packet */
     .parse_packet   = garmin_ser_parse,	/* parse message packets */
     .rtcm_writer    = NULL,		/* don't send DGPS corrections */
+    .init_query     = NULL,		/* non-perturbing initial query */
     .event_hook     = garmin_event_hook,/* lifetime ebent handler */
 #ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
@@ -1443,6 +1446,7 @@ const struct gps_type_t driver_garmin_ser_binary =
     .get_packet     = generic_get,       /* how to grab a packet */
     .parse_packet   = garmin_ser_parse,	/* parse message packets */
     .rtcm_writer    = NULL,		/* don't send DGPS corrections */
+    .init_query     = NULL,		/* non-perturbing initial query */
     .event_hook     = NULL,	        /* lifetime event handler */
 #ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,		/* no speed switcher */
