@@ -7,6 +7,7 @@
 #include <stdbool.h>
 #include <string.h>
 #include <stdarg.h>
+#include <assert.h>
 #include <ctype.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
@@ -1424,6 +1425,15 @@ static void path_rewrite(struct gps_device_t *session, char *prefix)
      * from the device.
      */
     char *prefloc;
+
+#ifdef S_SPLINT_S
+    assert(prefix != NULL && session->lexer.outbuffer != NULL);
+#endif /* S_SPLINT_S */
+
+    /* possibly the rewrite has been done already, this comw up in gpsmon */
+    if (strstr((char *)session->lexer.outbuffer, session->gpsdata.dev.path) != NULL)
+	return;
+
     for (prefloc = (char *)session->lexer.outbuffer;
 	 prefloc < (char *)session->lexer.outbuffer+session->lexer.outbuflen;
 	 prefloc++)
@@ -1450,7 +1460,7 @@ static gps_mask_t json_pass_packet(struct gps_device_t *session)
     gpsd_report(&session->context->errout, LOG_IO,
 		"<= GPS: %s\n", (char *)session->lexer.outbuffer);
 
-    if (!str_starts_with(session->gpsdata.dev.path, "gpsd://localhost:"))
+    if (strstr(session->gpsdata.dev.path, ":/") != NULL && strstr(session->gpsdata.dev.path, "localhost") == NULL)
     {
 	/*@-nullpass@*/ /* required only because splint is buggy */
 	/* devices and paths need to be edited */
