@@ -21,7 +21,6 @@ extern "C" {
 #ifndef S_SPLINT_S
 #include <pthread.h>	/* pacifies OpenBSD's compiler */
 #endif
-#include "compiler.h"
 
 /*
  * 4.1 - Base version for initial JSON protocol (Dec 2009, release 2.90)
@@ -1884,7 +1883,7 @@ struct policy_t {
     char remote[GPS_PATH_MAX];		/* ...if this was passthrough */
 };
 
-struct timedrift_t {
+struct timedelta_t {
     struct timespec	real;
     struct timespec	clock;
 };
@@ -1955,8 +1954,8 @@ struct gps_data_t {
 #define POLICY_SET	(1llu<<29)
 #define LOGMESSAGE_SET	(1llu<<30)
 #define ERROR_SET	(1llu<<31)
-#define TIMEDRIFT_SET	(1llu<<32)
-#define PPSDRIFT_SET	(1llu<<33)
+#define TOFF_SET	(1llu<<32)	/* not yet used */
+#define PPS_SET 	(1llu<<33)
 #define SET_HIGH_BIT	34
     timestamp_t online;		/* NZ if GPS is on line, 0 if not.
 				 *
@@ -2006,7 +2005,7 @@ struct gps_data_t {
     } devices;
 
     /* pack things never reported together to reduce structure size */
-#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|ATTITUDE_SET|GST_SET|VERSION_SET|LOGMESSAGE_SET|ERROR_SET|TIMEDRIFT_SET)
+#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|ATTITUDE_SET|GST_SET|VERSION_SET|LOGMESSAGE_SET|ERROR_SET|TOFF_SET|PPS_SET)
     union {
 	/* unusual forms of sensor data that might come up the pipe */
 	struct rtcm2_t	rtcm2;
@@ -2019,7 +2018,8 @@ struct gps_data_t {
 	/* "artificial" structures for various protocol responses */
 	struct version_t version;
 	char error[256];
-	struct timedrift_t timedrift;
+	struct timedelta_t toff;
+	struct timedelta_t pps;
     };
 
     /* Private data - client code must not set this */
@@ -2039,6 +2039,8 @@ extern int gps_mainloop(struct gps_data_t *, int,
 extern const char /*@null observer@*/ *gps_data(const struct gps_data_t *);
 extern const char /*@observer@*/ *gps_errstr(const int);
 
+int json_toff_read(const char *buf, struct gps_data_t *,
+		  /*@null@*/ const char **);
 int json_pps_read(const char *buf, struct gps_data_t *,
 		  /*@null@*/ const char **);
 
