@@ -22,6 +22,11 @@
 #include "gpsd.h"
 #include "revision.h"
 
+#ifdef SHM_EXPORT_ENABLE
+#include <sys/ipc.h>
+#include <sys/shm.h>
+#endif /* SHM_EXPORT_ENABLE */
+
 #define HIGH_LEVEL_TIMEOUT	8
 
 static int debuglevel;
@@ -30,31 +35,13 @@ static unsigned int timeout = 0;	/* no timeout */
 static struct gps_context_t context;
 static bool hunting = true;
 
-ssize_t gpsd_write(struct gps_device_t *session,
-		   const char *buf,
-		   const size_t len)
-/* pass low-level data to devices straight through */
-{
-    return gpsd_serial_write(session, buf, len);
-}
-
 /*
  * Set this as high or higher than the maximum number of subtype
  * probes in drivers.c.
  */
 #define REDIRECT_SNIFF	15
 
-void gpsd_report(const int debuglevel, const int errlevel,
-		 const char *fmt, ...)
-{
-    va_list ap;
-
-    va_start(ap, fmt);
-    gpsd_labeled_report(debuglevel, errlevel, "gpsctl:", fmt, ap);
-    va_end(ap);
-			
-}
-
+#if defined(RECONFIGURE_ENABLE) || defined(CONTROLSEND_ENABLE)
 static void settle(struct gps_device_t *session)
 /* allow the device to settle after a control operation */
 {
@@ -65,6 +52,7 @@ static void settle(struct gps_device_t *session)
     (void)usleep(50000);
     (void)tcdrain(session->gpsdata.gps_fd);
 }
+#endif /* defined(RECONFIGURE_ENABLE) || defined(CONTROLSEND_ENABLE) */
 
 /*
  * Allows any response other than ERROR.  Use it for queries where a 
@@ -101,21 +89,21 @@ static bool gps_query(/*@out@*/struct gps_data_t *gpsdata,
     (void)vsnprintf(buf, sizeof(buf)-2, fmt, ap);
     va_end(ap);
     if (buf[strlen(buf)-1] != '\n')
-	(void)strlcat(buf, "\n", BUFSIZ);
+	(void)strlcat(buf, "\n", sizeof(buf));
     /*@-usedef@*/
     if (write(gpsdata->gps_fd, buf, strlen(buf)) <= 0) {
-	gpsd_report(context.debug, LOG_ERROR, "gps_query(), write failed\n");
+	gpsd_report(&context.errout, LOG_ERROR, "gps_query(), write failed\n");
 	return false;
     }
     /*@+usedef@*/
-    gpsd_report(context.debug, LOG_PROG, "gps_query(), wrote, %s\n", buf);
+    gpsd_report(&context.errout, LOG_PROG, "gps_query(), wrote, %s\n", buf);
 
     FD_ZERO(&rfds);
     starttime = time(NULL);
     for (;;) {
 	FD_CLR(gpsdata->gps_fd, &rfds);
 
-	gpsd_report(context.debug, LOG_PROG, "waiting...\n");
+	gpsd_report(&context.errout, LOG_PROG, "waiting...\n");
 
 	/*@ -usedef -type -nullpass -compdef @*/
 	tv.tv_sec = 2;
@@ -128,16 +116,16 @@ static bool gps_query(/*@out@*/struct gps_data_t *gpsdata,
 #endif
 	    if (errno == EINTR || !FD_ISSET(gpsdata->gps_fd, &rfds))
 		continue;
-	    gpsd_report(context.debug, LOG_ERROR, "select %s\n", strerror(errno));
+	    gpsd_report(&context.errout, LOG_ERROR, "select %s\n", strerror(errno));
 	    exit(EXIT_FAILURE);
 	}
 	/*@ +usedef +type +nullpass +compdef @*/
 
-	gpsd_report(context.debug, LOG_PROG, "reading...\n");
+	gpsd_report(&context.errout, LOG_PROG, "reading...\n");
 
 	(void)gps_read(gpsdata);
 	if (ERROR_SET & gpsdata->set) {
-	    gpsd_report(context.debug, LOG_ERROR, "error '%s'\n", gpsdata->error);
+	    gpsd_report(&context.errout, LOG_ERROR, "error '%s'\n", gpsdata->error);
 	    return false;
 	}
 
@@ -145,7 +133,7 @@ static bool gps_query(/*@out@*/struct gps_data_t *gpsdata,
 	if ((expect == NON_ERROR) || (expect & gpsdata->set) != 0)
 	    return true;
 	else if (timeout > 0 && (time(NULL) - starttime > timeout)) {
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"timed out after %d seconds\n",
 			timeout);
 	    return false;
@@ -159,10 +147,10 @@ static bool gps_query(/*@out@*/struct gps_data_t *gpsdata,
 static void onsig(int sig)
 {
     if (sig == SIGALRM) {
-	gpsd_report(context.debug, LOG_ERROR, "packet recognition timed out.\n");
+	gpsd_report(&context.errout, LOG_ERROR, "packet recognition timed out.\n");
 	exit(EXIT_FAILURE);
     } else {
-	gpsd_report(context.debug, LOG_ERROR, "killed by signal %d\n", sig);
+	gpsd_report(&context.errout, LOG_ERROR, "killed by signal %d\n", sig);
 	exit(EXIT_SUCCESS);
     }
 }
@@ -199,6 +187,7 @@ static void ctlhook(struct gps_device_t *device UNUSED, gps_mask_t changed UNUSE
     }
 }
 
+/*@-mustfreeonly -observertrans -statictrans@*/
 int main(int argc, char **argv)
 {
     int option, status;
@@ -214,8 +203,10 @@ int main(int argc, char **argv)
     ssize_t cooklen = 0;
 #endif /* RECONFIGURE_ENABLE */
 
-#define USAGE	"usage: gpsctl [-l] [-b | -n | -r] [-D n] [-s speed] [-c rate] [-T timeout] [-V] [-t devtype] [-x control] [-e] <device>\n"
-    while ((option = getopt(argc, argv, "bec:fhlnrs:t:x:D:T:V")) != -1) {
+    context.errout.label = "gpsctl";
+
+#define USAGE	"usage: gpsctl [-l] [-b | -n | -r] [-D n] [-s speed] [-c rate] [-T timeout] [-V] [-t devtype] [-x control] [-R] [-e] [device]\n"
+    while ((option = getopt(argc, argv, "bec:fhlnrs:t:x:D:RT:V")) != -1) {
 	switch (option) {
 	case 'b':		/* switch to vendor binary mode */
 	    to_binary = true;
@@ -224,7 +215,7 @@ int main(int argc, char **argv)
 #ifdef RECONFIGURE_ENABLE
 	    rate = optarg;
 #else
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"cycle-change capability has been conditioned out.\n");
 #endif /* RECONFIGURE_ENABLE */
 	    break;
@@ -233,12 +224,12 @@ int main(int argc, char **argv)
 	    control = optarg;
 	    lowlevel = true;
 	    if ((cooklen = hex_escapes(cooked, control)) <= 0) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "invalid escape string (error %d)\n", (int)cooklen);
 		exit(EXIT_FAILURE);
 	    }
 #else
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"control_send capability has been conditioned out.\n");
 #endif /* CONTROLSEND_ENABLE */
 	    break;
@@ -278,7 +269,7 @@ int main(int argc, char **argv)
 #ifdef RECONFIGURE_ENABLE
 	    to_nmea = true;
 #else
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"speed-change capability has been conditioned out.\n");
 #endif /* RECONFIGURE_ENABLE */
 	    break;
@@ -287,7 +278,7 @@ int main(int argc, char **argv)
 	    reset = true;
 	    lowlevel = false;	/* so we'll abort if the daemon is running */
 #else
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"reset capability has been conditioned out.\n");
 #endif /* RECONFIGURE_ENABLE */
 	    break;
@@ -295,13 +286,33 @@ int main(int argc, char **argv)
 #ifdef RECONFIGURE_ENABLE
 	    speed = optarg;
 #else
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"speed-change capability has been conditioned out.\n");
 #endif /* RECONFIGURE_ENABLE */
 	    break;
 	case 't':		/* force the device type */
 	    devtype = optarg;
 	    break;
+	case 'R':		/* remove the SHM export segment */
+#ifdef SHM_EXPORT_ENABLE
+	    /*@-nullpass@*/
+	    status = shmget(getenv("GPSD_SHM_KEY") ? (key_t)strtol(getenv("GPSD_SHM_KEY"), NULL, 0) : (key_t)GPSD_SHM_KEY, 0, 0);
+	    if (status == -1) {
+		gpsd_report(&context.errout, LOG_WARN,
+			    "GPSD SHM segment does not exist.\n");
+		exit(1);
+	    } else {
+		status = shmctl(status, IPC_RMID, NULL);
+		if (status == -1) {
+		    gpsd_report(&context.errout, LOG_ERROR,
+				"shmctl failed, errno = %d (%s)\n",
+				errno, strerror(errno));
+		    exit(1);
+		}
+	    }
+	    /*@+nullpass@*/
+	    exit(0);
+#endif /* SHM_EXPORT_ENABLE */
 	case 'T':		/* set the timeout on packet recognition */
 	    timeout = (unsigned)atoi(optarg);
 	    explicit_timeout = true;
@@ -335,22 +346,22 @@ int main(int argc, char **argv)
 	    }
 	}
 	if (matchcount == 0)
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"no driver type name matches '%s'.\n", devtype);
 	else if (matchcount == 1) {
 	    assert(forcetype != NULL);
-	    gpsd_report(context.debug, LOG_PROG,
+	    gpsd_report(&context.errout, LOG_PROG,
 			"%s driver selected.\n", forcetype->type_name);
 	} else {
 	    forcetype = NULL;
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"%d driver type names match '%s'.\n",
 			matchcount, devtype);
 	}
     }
 
     if (((int)to_nmea + (int)to_binary + (int)reset) > 1) {
-	gpsd_report(context.debug, LOG_ERROR, "make up your mind, would you?\n");
+	gpsd_report(&context.errout, LOG_ERROR, "make up your mind, would you?\n");
 	exit(EXIT_SUCCESS);
     }
 
@@ -362,7 +373,7 @@ int main(int argc, char **argv)
     if (!lowlevel) {
 	/* Try to open the stream to gpsd. */
 	if (gps_open(NULL, NULL, &gpsdata) != 0) {
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"no gpsd running or network error: %s.\n",
 			gps_errstr(errno));
 	    lowlevel = true;
@@ -378,21 +389,21 @@ int main(int argc, char **argv)
 
 	/* what devices have we available? */
 	if (!gps_query(&gpsdata, DEVICELIST_SET, (int)timeout, "?DEVICES;\n")) {
-	    gpsd_report(context.debug, LOG_ERROR, "no DEVICES response received.\n");
+	    gpsd_report(&context.errout, LOG_ERROR, "no DEVICES response received.\n");
 	    (void)gps_close(&gpsdata);
 	    exit(EXIT_FAILURE);
 	}
 	if (gpsdata.devices.ndevices == 0) {
-	    gpsd_report(context.debug, LOG_ERROR, "no devices connected.\n");
+	    gpsd_report(&context.errout, LOG_ERROR, "no devices connected.\n");
 	    (void)gps_close(&gpsdata);
 	    exit(EXIT_FAILURE);
 	} else if (gpsdata.devices.ndevices > 1 && device == NULL) {
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"multiple devices and no device specified.\n");
 	    (void)gps_close(&gpsdata);
 	    exit(EXIT_FAILURE);
 	}
-	gpsd_report(context.debug, LOG_PROG,
+	gpsd_report(&context.errout, LOG_PROG,
 		    "%d device(s) found.\n",gpsdata.devices.ndevices);
 
 	/* try to mine the devicelist return for the data we want */
@@ -405,7 +416,7 @@ int main(int argc, char **argv)
 		if (strcmp(device, gpsdata.devices.list[i].path) == 0) {
 		    goto devicelist_entry_matches;
 		}
-	    gpsd_report(context.debug, LOG_ERROR,
+	    gpsd_report(&context.errout, LOG_ERROR,
 			"specified device not found in device list.\n");
 	    (void)gps_close(&gpsdata);
 	    exit(EXIT_FAILURE);
@@ -419,7 +430,7 @@ int main(int argc, char **argv)
 	/* if the device has not identified, watch it until it does so */
 	if (gpsdata.dev.driver[0] == '\0') {
 	    if (gps_stream(&gpsdata, WATCH_ENABLE|WATCH_JSON, NULL) == -1) {
-		gpsd_report(context.debug, LOG_ERROR, "stream set failed.\n");
+		gpsd_report(&context.errout, LOG_ERROR, "stream set failed.\n");
 		(void)gps_close(&gpsdata);
 		exit(EXIT_FAILURE);
 	    }
@@ -427,7 +438,7 @@ int main(int argc, char **argv)
 	    while (devcount > 0) {
 		errno = 0;
 		if (gps_read(&gpsdata) == -1) {
-		    gpsd_report(context.debug, LOG_ERROR, "data read failed.\n");
+		    gpsd_report(&context.errout, LOG_ERROR, "data read failed.\n");
 		    (void)gps_close(&gpsdata);
 		    exit(EXIT_FAILURE);
 		}
@@ -440,7 +451,7 @@ int main(int argc, char **argv)
 		    }
 		}
 	    }
-	    gpsd_report(context.debug, LOG_ERROR, "data read failed.\n");
+	    gpsd_report(&context.errout, LOG_ERROR, "data read failed.\n");
 	    (void)gps_close(&gpsdata);
 	    exit(EXIT_FAILURE);
 	matching_device_seen:;
@@ -448,7 +459,7 @@ int main(int argc, char **argv)
 
 	/* sanity check */
 	if (gpsdata.dev.driver[0] == '\0') {
-	    gpsd_report(context.debug, LOG_SHOUT, "%s can't be identified.\n",
+	    gpsd_report(&context.errout, LOG_SHOUT, "%s can't be identified.\n",
 			gpsdata.dev.path);
 	    (void)gps_close(&gpsdata);
 	    exit(EXIT_SUCCESS);
@@ -472,7 +483,7 @@ int main(int argc, char **argv)
 #ifdef RECONFIGURE_ENABLE
 	if (reset)
 	{
-	    gpsd_report(context.debug, LOG_PROG,
+	    gpsd_report(&context.errout, LOG_PROG,
 			"cannot reset with gpsd running.\n");
 	    exit(EXIT_SUCCESS);
 	}
@@ -490,24 +501,24 @@ int main(int argc, char **argv)
 	    if (!gps_query(&gpsdata, NON_ERROR, (int)timeout, 
 			   "?DEVICE={\"path\":\"%s\",\"native\":0}\r\n",
 			   device)) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "%s mode change to NMEA failed\n",
 			    gpsdata.dev.path);
 		status = 1;
 	    } else
-		gpsd_report(context.debug, LOG_PROG,
+		gpsd_report(&context.errout, LOG_PROG,
 			    "%s mode change succeeded\n", gpsdata.dev.path);
 	}
 	else if (to_binary) {
 	    if (!gps_query(&gpsdata, NON_ERROR, (int)timeout,
 			   "?DEVICE={\"path\":\"%s\",\"native\":1}\r\n",
 			   device)) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "%s mode change to native mode failed\n",
 			    gpsdata.dev.path);
 		status = 1;
 	    } else
-		gpsd_report(context.debug, LOG_PROG,
+		gpsd_report(&context.errout, LOG_PROG,
 			    "%s mode change succeeded\n",
 			    gpsdata.dev.path);
 	}
@@ -527,19 +538,19 @@ int main(int argc, char **argv)
 		if (modespec!=NULL) {
 		    *modespec = '\0';
 		    if (strchr("78", *++modespec) == NULL) {
-			gpsd_report(context.debug, LOG_ERROR,
+			gpsd_report(&context.errout, LOG_ERROR,
 				    "No support for that word length.\n");
 			status = 1;
 		    }
 		    parity = *++modespec;
 		    if (strchr("NOE", parity) == NULL) {
-			gpsd_report(context.debug, LOG_ERROR,
+			gpsd_report(&context.errout, LOG_ERROR,
 				    "What parity is '%c'?\n", parity);
 			status = 1;
 		    }
 		    stopbits = *++modespec;
 		    if (strchr("12", stopbits) == NULL) {
-			gpsd_report(context.debug, LOG_ERROR,
+			gpsd_report(&context.errout, LOG_ERROR,
 				    "Stop bits must be 1 or 2.\n");
 			status = 1;
 		    }
@@ -551,13 +562,13 @@ int main(int argc, char **argv)
 				     device, speed, parity, stopbits);
 	    }
 	    if (atoi(speed) != (int)gpsdata.dev.baudrate) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "%s driver won't support %s%c%c\n",
 			    gpsdata.dev.path,
 			    speed, parity, stopbits);
 		status = 1;
 	    } else
-		gpsd_report(context.debug, LOG_PROG,
+		gpsd_report(&context.errout, LOG_PROG,
 			    "%s change to %s%c%c succeeded\n",
 			    gpsdata.dev.path,
 			    speed, parity, stopbits);
@@ -579,14 +590,14 @@ int main(int argc, char **argv)
 	int i;
 
 	if (device == NULL || forcetype == NULL) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "device and type must be specified for the reset operation.\n");
 		exit(EXIT_FAILURE);
 	    }
 
 	/*@ -mustfreeonly -immediatetrans @*/
-	gps_context_init(&context);
-	context.debug = debuglevel;
+	gps_context_init(&context, "gpsctl");
+	context.errout.debug = debuglevel;
 	session.context = &context;
 	gpsd_tty_init(&session);
 	(void)strlcpy(session.gpsdata.dev.path, device, sizeof(session.gpsdata.dev.path));
@@ -621,8 +632,8 @@ int main(int argc, char **argv)
 	 */
 
 	/*@ -mustfreeonly -immediatetrans @*/
-	gps_context_init(&context);
-	context.debug = debuglevel;
+	gps_context_init(&context, "gpsctl");
+	context.errout.debug = debuglevel;
 	session.context = &context;	/* in case gps_init isn't called */
 
 	if (echo)
@@ -640,19 +651,19 @@ int main(int argc, char **argv)
 	if (!(forcetype != NULL && echo)) {
 	    int maxfd = 0;
 	    if (device == NULL) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "device must be specified for low-level access.\n");
 		exit(EXIT_FAILURE);
 	    }
 
 	    gpsd_init(&session, &context, device);
 	    if (gpsd_activate(&session, O_PROBEONLY) < 0) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "initial GPS device %s open failed\n",
 			    device);
 		exit(EXIT_FAILURE);
 	    }
-	    gpsd_report(context.debug, LOG_INF, 
+	    gpsd_report(&context.errout, LOG_INF, 
 			"device %s activated\n", session.gpsdata.dev.path);
 	    /*@i1@*/FD_SET(session.gpsdata.gps_fd, &all_fds);
 	    if (session.gpsdata.gps_fd > maxfd)
@@ -665,19 +676,15 @@ int main(int argc, char **argv)
 	    /* grab packets until we time out, get sync, or fail sync */
 	    for (hunting = true; hunting; )
 	    {
-#ifdef EFDS
 		fd_set efds;
-#endif /* EFDS */
-		switch(gpsd_await_data(&rfds, maxfd, &all_fds, context.debug))
+		switch(gpsd_await_data(&rfds, &efds, maxfd, &all_fds, &context.errout))
 		{
 		case AWAIT_GOT_INPUT:
 		    break;
 		case AWAIT_NOT_READY:
-#ifdef EFDS
 		    /* no recovery from bad fd is possible */
 		    if (FD_ISSET(session.gpsdata.gps_fd, &efds))
 			exit(EXIT_FAILURE);
-#endif /* EFDS */
 		    continue;
 		case AWAIT_FAILED:
 		    exit(EXIT_FAILURE);
@@ -694,11 +701,11 @@ int main(int argc, char **argv)
 		    break;
 		case DEVICE_ERROR:
 		    /* this is where a failure to sync lands */
-		    gpsd_report(context.debug, LOG_WARN,
+		    gpsd_report(&context.errout, LOG_WARN,
 				"device error, bailing out.\n");
 		    exit(EXIT_FAILURE);
 		case DEVICE_EOF:
-		    gpsd_report(context.debug, LOG_WARN,
+		    gpsd_report(&context.errout, LOG_WARN,
 				"device signed off, bailing out.\n");
 		    exit(EXIT_SUCCESS);
 		default:
@@ -707,13 +714,13 @@ int main(int argc, char **argv)
 	    }
 	    /*@+compdef@*/
 
-	    gpsd_report(context.debug, LOG_PROG,
+	    gpsd_report(&context.errout, LOG_PROG,
 			"%s looks like a %s at %d.\n",
 			device, gpsd_id(&session),
 			session.gpsdata.dev.baudrate);
 
 	    if (forcetype!=NULL && strcmp("NMEA0183", session.device_type->type_name) !=0 && strcmp(forcetype->type_name, session.device_type->type_name)!=0) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			    "'%s' doesn't match non-generic type '%s' of selected device.\n",
 			    forcetype->type_name,
 			    session.device_type->type_name);
@@ -744,14 +751,14 @@ int main(int argc, char **argv)
 	    bool write_enable = context.readonly;
 	    context.readonly = false;
 	    if (session.device_type->mode_switcher == NULL) {
-		gpsd_report(context.debug, LOG_SHOUT,
+		gpsd_report(&context.errout, LOG_SHOUT,
 			      "%s devices have no mode switch.\n",
 			      session.device_type->type_name);
 		status = 1;
 	    } else {
 		int target_mode = to_nmea ? MODE_NMEA : MODE_BINARY;
 
-		gpsd_report(context.debug, LOG_SHOUT,
+		gpsd_report(&context.errout, LOG_SHOUT,
 			    "switching to mode %s.\n",
 			    to_nmea ? "NMEA" : "BINARY");
 		session.device_type->mode_switcher(&session, target_mode);
@@ -770,19 +777,19 @@ int main(int argc, char **argv)
 	    if (modespec!=NULL) {
 		*modespec = '\0';
 		if (strchr("78", *++modespec) == NULL) {
-		    gpsd_report(context.debug, LOG_ERROR,
+		    gpsd_report(&context.errout, LOG_ERROR,
 				"No support for that word lengths.\n");
 		    status = 1;
 		}
 		parity = *++modespec;
 		if (strchr("NOE", parity) == NULL) {
-		    gpsd_report(context.debug, LOG_ERROR,
+		    gpsd_report(&context.errout, LOG_ERROR,
 				"What parity is '%c'?\n", parity);
 		    status = 1;
 		}
 		stopbits = *++modespec;
 		if (strchr("12", parity) == NULL) {
-		    gpsd_report(context.debug, LOG_ERROR,
+		    gpsd_report(&context.errout, LOG_ERROR,
 				"Stop bits must be 1 or 2.\n");
 		    status = 1;
 		}
@@ -790,7 +797,7 @@ int main(int argc, char **argv)
 	    }
 	    if (status == 0) {
 		if (session.device_type->speed_switcher == NULL) {
-		    gpsd_report(context.debug, LOG_ERROR,
+		    gpsd_report(&context.errout, LOG_ERROR,
 				"%s devices have no speed switch.\n",
 				session.device_type->type_name);
 		    status = 1;
@@ -800,12 +807,12 @@ int main(int argc, char **argv)
 							     parity,
 							     stopbits)) {
 		    settle(&session);
-		    gpsd_report(context.debug, LOG_PROG,
+		    gpsd_report(&context.errout, LOG_PROG,
 				"%s change to %s%c%d succeeded\n",
 				session.gpsdata.dev.path,
 				speed, parity, stopbits);
 		} else {
-		    gpsd_report(context.debug, LOG_ERROR,
+		    gpsd_report(&context.errout, LOG_ERROR,
 				"%s driver won't support %s%c%d.\n",
 				session.gpsdata.dev.path,
 				speed, parity, stopbits);
@@ -817,7 +824,7 @@ int main(int argc, char **argv)
 	    bool write_enable = context.readonly;
 	    context.readonly = false;
 	    if (session.device_type->rate_switcher == NULL) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			      "%s devices have no rate switcher.\n",
 			      session.device_type->type_name);
 		status = 1;
@@ -825,7 +832,7 @@ int main(int argc, char **argv)
 		double rate_dbl = strtod(rate, NULL);
 
 		if (!session.device_type->rate_switcher(&session, rate_dbl)) {
-		    gpsd_report(context.debug, LOG_ERROR, "rate switch failed.\n");
+		    gpsd_report(&context.errout, LOG_ERROR, "rate switch failed.\n");
 		    status = 1;
 		}
 		settle(&session);
@@ -839,7 +846,7 @@ int main(int argc, char **argv)
 	    bool write_enable = context.readonly;
 	    context.readonly = false;
 	    if (session.device_type->control_send == NULL) {
-		gpsd_report(context.debug, LOG_ERROR,
+		gpsd_report(&context.errout, LOG_ERROR,
 			      "%s devices have no control sender.\n",
 			      session.device_type->type_name);
 		status = 1;
@@ -847,7 +854,7 @@ int main(int argc, char **argv)
 		if (session.device_type->control_send(&session,
 						      cooked,
 						      (size_t)cooklen) == -1) {
-		    gpsd_report(context.debug, LOG_ERROR,
+		    gpsd_report(&context.errout, LOG_ERROR,
 				"control transmission failed.\n");
 		    status = 1;
 		}
@@ -863,3 +870,6 @@ int main(int argc, char **argv)
 	/*@ +mustfreeonly +immediatetrans @*/
     }
 }
+/*@+mustfreeonly +observertrans +statictrans@*/
+
+/* end */
