@@ -1500,7 +1500,7 @@ static void all_reports(struct gps_device_t *device, gps_mask_t changed)
     }
 
 
-#ifdef NTPSHM_ENABLE
+#ifdef NTP_ENABLE
     /*
      * Time is eligible for shipping to NTPD if the driver has
      * asserted PPSTIME_IS at any point in the current cycle.
@@ -1521,15 +1521,30 @@ static void all_reports(struct gps_device_t *device, gps_mask_t changed)
 	//gpsd_report(&context.errout, LOG_PROG, "NTP: Not a new time\n");
     } else if (!device->ship_to_ntpd) {
 	//gpsd_report(&context.errout, LOG_PROG, "NTP: No precision time report\n");
-    } else if (device->shm_clock != NULL) {
-	/*@-compdef@*/
-	struct timedrift_t td;
-	ntpshm_latch(device, &td);
-	(void)ntpshm_put(device, device->shm_clock, &td);
-	/*@+compdef@*/
-    }
+    } else {
+	struct timedelta_t td;
+	ntp_latch(device, &td);
+#ifdef NTPSHM_ENABLE
+	if (device->shm_clock != NULL) {
+	    /*@-compdef@*/
+	    (void)ntpshm_put(device, device->shm_clock, &td);
+	    /*@+compdef@*/
+	}
 #endif /* NTPSHM_ENABLE */
 
+#ifdef SOCKET_EXPORT_ENABLE
+	/*@-type@*//* splint is confused about struct timespec */
+	notify_watchers(device, false, true,
+			"{\"class\":\"TOFF\",\"device\":\"%s\",\"real_sec\":%ld, \"real_nsec\":%ld,\"clock_sec\":%ld,\"clock_nsec\":%ld}\r\n",
+			device->gpsdata.dev.path,
+			td.real.tv_sec, td.real.tv_nsec,
+			td.clock.tv_sec, td.clock.tv_nsec);
+	/*@+type@*/
+#endif /* SOCKET_EXPORT_ENABLE */
+
+    }
+#endif /* NTP_ENABLE */
+
     /*
      * If no reliable end of cycle, must report every time
      * a sentence changes position or mode. Likely to
@@ -1649,10 +1664,10 @@ static int handle_gpsd_request(struct subscriber_t *sub, const char *buf)
 }
 #endif /* SOCKET_EXPORT_ENABLE */
 
-#if defined(PPS_ENABLE) && defined(SOCKET_EXPORT_ENABLE)
-static void ship_pps_drift_message(struct gps_device_t *session,
-				   struct timedrift_t *td)
-/* on PPS interrupt, ship a drift message to all clients */
+#if defined(CONTROL_SOCKET_ENABLE) && defined(PPS_ENABLE) && defined(SOCKET_EXPORT_ENABLE)
+static void ship_pps_message(struct gps_device_t *session,
+				   struct timedelta_t *td)
+/* on PPS interrupt, ship a message to all clients */
 {
     /*@-type@*//* splint is confused about struct timespec */
     notify_watchers(session, true, true,
@@ -1798,7 +1813,7 @@ int main(int argc, char *argv[])
 #ifdef CONTROL_SOCKET_ENABLE
     INVALIDATE_SOCKET(csock);
 #if defined(PPS_ENABLE) && defined(SOCKET_EXPORT_ENABLE)
-    context.pps_hook = ship_pps_drift_message;
+    context.pps_hook = ship_pps_message;
 #endif /* PPS_ENABLE && SOCKET_EXPORT_ENABLE */
 #endif /* CONTROL_SOCKET_ENABLE */
 
