@@ -59,6 +59,9 @@ static char *type_name;
 static size_t promptlen = 0;
 struct termios cooked, rare;
 struct fixsource_t source;
+#ifdef NTP_ENABLE
+struct timedelta_t time_offset;
+#endif /* NTP_ENABLE */
 
 #ifdef PASSTHROUGH_ENABLE
 /* no methods, it's all device window */
@@ -165,6 +168,7 @@ static inline void report_unlock(void) { }
  *
  ******************************************************************************/
 
+#ifdef PPS_ENABLE
 static void visibilize(/*@out@*/char *buf2, size_t len2, const char *buf)
 /* string is mostly printable, dress up the nonprintables a bit */
 {
@@ -179,6 +183,7 @@ static void visibilize(/*@out@*/char *buf2, size_t len2, const char *buf)
 	    (void)snprintf(buf2 + strlen(buf2), 6, "\\x%02x",
 			   (unsigned)(*sp & 0xff));
 }
+#endif /* PPS_ENABLE */
 
 /*@-compdef -mustdefine@*/
 static void cond_hexdump(/*@out@*/char *buf2, size_t len2, 
@@ -209,6 +214,64 @@ static void cond_hexdump(/*@out@*/char *buf2, size_t len2,
 }
 /*@+compdef +mustdefine@*/
 
+#ifdef NTP_ENABLE
+/*@-compdef@*/
+/*@-type -noeffect@*/ /* splint is confused about struct timespec */
+void toff_update(WINDOW *win, int y, int x)
+{
+    if (time_offset.real.tv_sec != 0)
+    {
+	/* NOTE: can not use double here due to precision requirements */
+	struct timespec timedelta;
+	(void)wmove(win, y, x);
+	(void)wclrtoeol(win);
+	TS_SUB(&timedelta, &time_offset.clock, &time_offset.real);
+	if ( 86400 < (long)labs(timedelta.tv_sec) ) {
+	    /* more than one day off, overflow */
+	    /* need a bigger field to show it */
+	    (void)mvwprintw(win, y, x, "> 1 day");
+	} else {
+	    char buf[TIMESPEC_LEN];
+	    timespec_str( &timedelta, buf, sizeof(buf) );
+	    (void)mvwprintw(win, y, x, "%s", buf);
+	}
+	monitor_fixframe(win);
+    }
+}
+/*@+type +noeffect@*/
+/*@+compdef@*/
+#endif /* NTP_ENABLE */
+
+#ifdef PPS_ENABLE
+void pps_update(WINDOW *win, int y, int x)
+{
+    /*@-compdef@*/
+    /*@-type -noeffect@*/ /* splint is confused about struct timespec */
+    struct timedelta_t ppstimes;
+
+    if (pps_thread_lastpps(&session, &ppstimes) > 0) {
+	/* NOTE: can not use double here due to precision requirements */
+	struct timespec timedelta;
+	(void)wmove(win, y, x);
+	(void)wclrtoeol(win);
+	TS_SUB( &timedelta, &ppstimes.clock, &ppstimes.real);
+        if ( 86400 < (long)labs(timedelta.tv_sec) ) {
+	    /* more than one day off, overflow */
+            /* need a bigger field to show it */
+	    (void)mvwprintw(win, y, x, "> 1 day");
+        } else {
+	    char buf[TIMESPEC_LEN];
+	    timespec_str( &timedelta, buf, sizeof(buf) );
+	    (void)mvwprintw(win, y, x, "%s", buf);
+        }
+	monitor_fixframe(win);
+	(void)wnoutrefresh(win);
+    }
+    /*@+type +noeffect@*/
+    /*@+compdef@*/
+}
+#endif /* PPS_ENABLE */
+
 /******************************************************************************
  *
  * Curses I/O
@@ -252,30 +315,32 @@ static void monitor_dump_send(/*@in@*/ const char *buf, size_t len)
 
 /*@-compdef@*/
 static void gpsmon_report(const char *buf)
+/* log to the packet window if curses is up, otherwise stdout */
+{
+    /* report locking is left to caller */
+    if (!curses_active)
+	(void)fputs(buf, stdout);
+    else if (packetwin != NULL)
+	(void)waddstr(packetwin, buf);
+    if (logfile != NULL)
+	(void)fputs(buf, logfile);
+}
+/*@+compdef@*/
+
+#ifdef PPS_ENABLE
+/*@-compdef@*/
+static void packet_vlog(char *buf, size_t len, const char *fmt, va_list ap)
 {
     char buf2[BUFSIZ];
 
     visibilize(buf2, sizeof(buf2), buf);
 
     report_lock();
-    if (!curses_active)
-	(void)fputs(buf2, stdout);
-    else if (packetwin != NULL)
-	(void)waddstr(packetwin, buf2);
-    if (logfile != NULL)
-	(void)fputs(buf2, logfile);
+    (void)vsnprintf(buf2 + strlen(buf2), len, fmt, ap);
+    gpsmon_report(buf2);
     report_unlock();
 }
 /*@+compdef@*/
-
-#ifdef PPS_ENABLE
-/*@-compdef@*/
-static void packet_vlog(/*@out@*/char *buf, size_t len, const char *fmt, va_list ap)
-{
-    (void)vsnprintf(buf + strlen(buf), len, fmt, ap);
-    gpsmon_report(buf);
-}
-/*@+compdef@*/
 #endif
 
 #ifdef RECONFIGURE_ENABLE
@@ -660,13 +725,33 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
 /* per-packet hook */
 {
     char buf[BUFSIZ];
-#ifdef NTPSHM_ENABLE
-    struct timedrift_t td;
-#endif /* NTPSHM_ENABLE */
 
-#if defined(SOCKET_EXPORT_ENABLE) && defined(PPS_ENABLE) && defined(CONTROL_SOCKET_ENABLE)
-    if (!serial && str_starts_with((char*)device->lexer.outbuffer, "{\"class\":\"PPS\","))
-    {
+#if defined(SOCKET_EXPORT_ENABLE) && defined(PPS_ENABLE)
+    if (!serial && str_starts_with((char*)device->lexer.outbuffer, "{\"class\":\"TOFF\",")) {
+	const char *end = NULL;
+	int status = json_toff_read((const char *)device->lexer.outbuffer,
+				   &session.gpsdata,
+				   &end);
+	if (status != 0) {
+	    /* FIXME: figure out why using json_error_string() core dumps */
+	    complain("Ill-formed TOFF packet: %d", status);
+	    return;
+	} else {
+	    /*@-type -noeffect@*/ /* splint is confused about struct timespec */
+	    if (!curses_active)
+		(void)fprintf(stderr,
+			      "TOFF=%ld.%09ld real=%ld.%09ld\n",
+			      (long)session.gpsdata.toff.clock.tv_sec,
+			      (long)session.gpsdata.toff.clock.tv_nsec,
+			      (long)session.gpsdata.toff.real.tv_sec,
+			      (long)session.gpsdata.toff.real.tv_nsec);
+	    /*@+type +noeffect@*/
+#ifdef NTP_ENABLE
+	    time_offset = session.gpsdata.toff;
+#endif /* NTP_ENABLE */
+	    return;
+	}
+    } else if (!serial && str_starts_with((char*)device->lexer.outbuffer, "{\"class\":\"PPS\",")) {
 	const char *end = NULL;
 	struct gps_data_t noclobber;
 	int status = json_pps_read((const char *)device->lexer.outbuffer,
@@ -675,27 +760,37 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
 	if (status != 0) {
 	    /* FIXME: figure out why using json_error_string() core dumps */
 	    complain("Ill-formed PPS packet: %d", status);
-	    buf[0] = '\0';
+	    return;
 	} else {
 	    /*@-type -noeffect@*/ /* splint is confused about struct timespec */
 	    struct timespec timedelta;
-	    TS_SUB( &timedelta, &noclobber.timedrift.real, 
-		&noclobber.timedrift.clock);
+	    char timedelta_str[TIMESPEC_LEN];
+
+	    TS_SUB( &timedelta, &noclobber.pps.real, &noclobber.pps.clock);
+	    timespec_str( &timedelta, timedelta_str, sizeof(timedelta_str) );
+
+	    if (!curses_active) {
+		char pps_clock_str[TIMESPEC_LEN];
+		char pps_real_str[TIMESPEC_LEN];
+
+		timespec_str( &noclobber.pps.clock, pps_clock_str,
+			sizeof(pps_clock_str) );
+		timespec_str( &noclobber.pps.real, pps_real_str,
+			sizeof(pps_real_str) );
 
-	    if (!curses_active)
 		(void)fprintf(stderr,
-			      "Drift clock=%ld.%09ld clock=%ld.%09ld offset=%ld.%09ld\n",
-			      (long)noclobber.timedrift.clock.tv_sec,
-			      (long)noclobber.timedrift.clock.tv_nsec,
-			      (long)noclobber.timedrift.real.tv_sec,
-			      (long)noclobber.timedrift.real.tv_nsec,
-			      (long)timedelta.tv_sec,
-                              (long)timedelta.tv_nsec);
+			      "PPS=%.20s clock=%.20s offset=%.20s\n",
+			      pps_clock_str,
+			      pps_real_str,
+                              timedelta_str);
+	    }
 	    /*@+type +noeffect@*/
 
-	    (void)strlcpy(buf, PPSBAR, sizeof(buf));
+	    (void)snprintf(buf, sizeof(buf), 
+			"------------------- PPS offset: %.20s ------\n ",
+			timedelta_str);
 	    /* coverity[missing_lock] */
-	    session.ppslast = noclobber.timedrift;
+	    session.ppslast = noclobber.pps;
 	    /* coverity[missing_lock] */
 	    session.ppscount++;
 	}
@@ -747,7 +842,7 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
 
     report_unlock();
 
-#ifdef NTPSHM_ENABLE
+#ifdef NTP_ENABLE
     /* Update the last fix time seen for PPS if we've actually seen one,
      * and it is a new second. */
     if ( 0 != isnan(device->newdata.time)) {
@@ -755,8 +850,8 @@ static void gpsmon_hook(struct gps_device_t *device, gps_mask_t changed UNUSED)
     } else if (device->newdata.time == device->last_fixtime.real) {
 	// "NTP: Not a new time
     } else 
-	ntpshm_latch(device, &td);
-#endif /* NTPSHM_ENABLE */
+	ntp_latch(device, &time_offset);
+#endif /* NTP_ENABLE */
 }
 /*@+observertrans +nullpass +globstate +compdef +uniondef@*/
 
@@ -1037,7 +1132,7 @@ static bool do_command(const char *line)
 
 #ifdef PPS_ENABLE
 static /*@observer@*/ char *pps_report(struct gps_device_t *session UNUSED,
-			struct timedrift_t *td UNUSED) {
+			struct timedelta_t *td UNUSED) {
     packet_log(PPSBAR);
     return "gpsmon";
 }
