@@ -23,6 +23,8 @@
  *
  */
 
+#include "gpsd_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -30,7 +32,9 @@
 #include <string.h>
 #include <strings.h>
 #include <fcntl.h>
+#ifdef HAVE_TERMIOS_H
 #include <termios.h>
+#endif /* HAVE_TERMIOS_H */
 #include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -342,11 +346,13 @@ int main(int argc, char **argv)
 		    switch( option_u ) {
 		    case 2:
 			(void)snprintf(tmstr_u, sizeof(tmstr_u), 
-				       " %ld.%06ld", now.tv_sec, now.tv_usec);
+				       " %ld.%06ld", 
+				       (long)now.tv_sec,
+				       (long)now.tv_usec);
 			break;
 		    case 1:
 			(void)snprintf(tmstr_u, sizeof(tmstr_u), 
-				       ".%06ld", now.tv_usec);
+				       ".%06ld", (long)now.tv_usec);
 			break;
 		    default:
 			*tmstr_u = '\0';
