@@ -14,6 +14,7 @@
 
 #include "gps.h"
 #include "gpsd_config.h"
+#include "compiler.h"
 #include "libgps.h"
 
 #if defined(DBUS_EXPORT_ENABLE) && !defined(S_SPLINT_S)
@@ -33,13 +34,13 @@ struct privdata_t
  */
 static struct gps_data_t *share_gpsdata;
 static DBusConnection *connection;
-static char gpsd_devname[BUFSIZ];
 
 static DBusHandlerResult handle_gps_fix(DBusMessage * message)
 {
     DBusError error;
     /* this packet format was designed before we split eph */
     double eph;
+    const char *gpsd_devname = NULL;
 
     dbus_error_init(&error);
 
