@@ -117,7 +117,11 @@ void gpsd_report(const struct gpsd_errout_t *errout,
 		 const char *fmt, ...)
 /* assemble msg in printf(3) style, use errout hook or syslog for delivery */
 {
-#ifndef SQUELCH_ENABLE
+#ifdef SQUELCH_ENABLE
+    (void)errout;
+    (void)errlevel;
+    (void)fmt;
+#else
     if (errout->debug >= errlevel) {
 	char buf[BUFSIZ], buf2[BUFSIZ];
 	char *err_str;
@@ -173,6 +177,8 @@ void gpsd_report(const struct gpsd_errout_t *errout,
 
 	if (getpid() == getsid(getpid()))
 	    syslog((errlevel == 0) ? LOG_ERR : LOG_NOTICE, "%s", buf2);
+	else if (errout->report != NULL)
+	    errout->report(buf2);
 	else
 	    (void)fputs(buf2, stderr);
 #if defined(PPS_ENABLE)
@@ -1643,8 +1649,8 @@ void gpsd_zero_satellites( /*@out@*/ struct gps_data_t *out)
 #endif
 }
 
-#ifdef NTPSHM_ENABLE
-void ntpshm_latch(struct gps_device_t *device, struct timedrift_t /*@out@*/*td)
+#ifdef NTP_ENABLE
+void ntp_latch(struct gps_device_t *device, struct timedelta_t /*@out@*/*td)
 /* latch the fact that we've saved a fix */
 {
     double fix_time, integral, fractional;
@@ -1660,12 +1666,15 @@ void ntpshm_latch(struct gps_device_t *device, struct timedrift_t /*@out@*/*td)
     TVTOTS(&td->clock, &clock_tv);
 #endif /* HAVE_CLOCK_GETTIME */
     fix_time = device->newdata.time;
+
+#ifdef TIMEHINT_ENABLE
     /* assume zero when there's no offset method */
     if (device->device_type == NULL
 	|| device->device_type->time_offset == NULL)
 	fix_time += 0.0;
     else
 	fix_time += device->device_type->time_offset(device);
+#endif /* TIMEHINT_ENABLE */
     /* it's ugly but timestamp_t is double */
     /* note loss of precision here
      * td->clock is in nanoSec
@@ -1684,6 +1693,6 @@ void ntpshm_latch(struct gps_device_t *device, struct timedrift_t /*@out@*/*td)
     /*@+compdef@*/
 #endif /* PPS_ENABLE */
 }
-#endif /* NTPSHM_ENABLE */
+#endif /* NTP_ENABLE */
 
 /* end */
