@@ -27,38 +27,7 @@
 #include "gpsd.h"
 
 #ifdef NTPSHM_ENABLE
-#include <sys/time.h>
-#include <sys/ipc.h>
-#include <sys/shm.h>
-
-#define NTPD_BASE	0x4e545030	/* "NTP0" */
-
-#define PPS_MIN_FIXES	3	/* # fixes to wait for before shipping PPS */
-
-/* the definition of shmTime is from ntpd source ntpd/refclock_shm.c */
-struct shmTime
-{
-    int mode;	/* 0 - if valid set
-		 *       use values,
-		 *       clear valid
-		 * 1 - if valid set
-		 *       if count before and after read of values is equal,
-		 *         use values
-		 *       clear valid
-		 */
-    volatile int count;
-    time_t clockTimeStampSec;
-    int clockTimeStampUSec;
-    time_t receiveTimeStampSec;
-    int receiveTimeStampUSec;
-    int leap;
-    int precision;
-    int nsamples;
-    volatile int valid;
-    unsigned        clockTimeStampNSec;     /* Unsigned ns timestamps */
-    unsigned        receiveTimeStampNSec;   /* Unsigned ns timestamps */
-    int             dummy[8];
-};
+#include "ntpshm.h"
 
 /* Note: you can start gpsd as non-root, and have it work with ntpd.
  * However, it will then only use the ntpshm segments 2 3, and higher.
@@ -139,6 +108,9 @@ struct shmTime
  * Removing these segments is usually not necessary, as the operating system 
  * garbage-collects them when they have no attached processes.
  */
+
+#define PPS_MIN_FIXES	3	/* # fixes to wait for before shipping PPS */
+
 static /*@null@*/ volatile struct shmTime *getShmTime(struct gps_context_t *context, int unit)
 {
     int shmid;
@@ -254,7 +226,7 @@ void ntpshm_session_init(struct gps_device_t *session)
     /*@+mustfreeonly@*/
 }
 
-int ntpshm_put(struct gps_device_t *session, volatile struct shmTime *shmseg, struct timedrift_t *td)
+int ntpshm_put(struct gps_device_t *session, volatile struct shmTime *shmseg, struct timedelta_t *td)
 /* put a received fix time into shared memory for NTP */
 {
     char real_str[TIMESPEC_LEN];
@@ -378,21 +350,27 @@ static void init_hook(struct gps_device_t *session)
 
 /* td is the real time and clock time of the edge */
 /* offset is actual_ts - clock_ts */
-static void chrony_send(struct gps_device_t *session, struct timedrift_t *td)
+static void chrony_send(struct gps_device_t *session, struct timedelta_t *td)
 {
     char real_str[TIMESPEC_LEN];
     char clock_str[TIMESPEC_LEN];
+    struct timespec offset;
     struct sock_sample sample;
 
     /* chrony expects tv-sec since Jan 1970 */
     sample.pulse = 0;
     sample.leap = session->context->leap_notify;
     sample.magic = SOCK_MAGIC;
-    /*@-type@*//* splint is confused about struct timespec */
-    TSTOTV(&sample.tv, &td->clock);
     /*@-compdef@*/
-    /* WARNING!  this will fail if timedelta more than a few seconds */
-    sample.offset = timespec_diff_ns(td->real, td->clock) / 1e9;
+    /*@-type@*//* splint is confused about struct timespec */
+    /* chronyd wants a timeval, not a timspec, not to worry, it is
+     * just the top of the second */
+    TSTOTV(&sample.tv, &td->clock);
+    /* calculate the offset as a timespec to not lose precision */
+    TS_SUB( &offset, &td->real, &td->clock);
+    /* if tv_sec greater than 2 then tv_nsec loses precision, but
+     * not a big deal as slewing will bbe required */
+    sample.offset = TSTONS( &offset );
     /*@+compdef@*/
 #ifdef __COVERITY__
     sample._pad = 0;
@@ -416,7 +394,7 @@ static void wrap_hook(struct gps_device_t *session)
 }
 
 static /*@observer@*/ char *report_hook(struct gps_device_t *session,
-					struct timedrift_t *td)
+					struct timedelta_t *td)
 /* ship the time of a PPS event to ntpd and/or chrony */
 {
     char *log1;
