@@ -625,12 +625,16 @@ static /*@null@*/ void *gpsd_ppsmonitor(void *arg)
 
 	if (ok) {
 	    /* offset is the skew from expected to observed pulse time */
-	    double offset;
+            struct timespec offset;
+	    /* offset as a printable string */
+	    char offset_str[TIMESPEC_LEN];
 	    /* delay after last fix */
-	    double delay;
+	    struct timespec  delay;
+	    /* delay as a printable string */
+	    char delay_str[TIMESPEC_LEN];
 	    char *log1 = NULL;
-	    /* drift.real is the time we think the pulse represents  */
-	    struct timedrift_t drift;
+	    /* ppstimes.real is the time we think the pulse represents  */
+	    struct timedelta_t ppstimes;
 	    gpsd_report(&session->context->errout, LOG_RAW,
 			"PPS edge accepted %.100s", log);
 #ifndef S_SPLINT_S
@@ -662,31 +666,40 @@ static /*@null@*/ void *gpsd_ppsmonitor(void *arg)
              */
 
 	    /*@+relaxtypes@*/
-	    drift.real.tv_sec = (time_t)trunc(last_fixtime_real) + 1;
-	    drift.real.tv_nsec = 0;  /* need to be fixed for 5Hz */
-	    drift.clock = clock_ts;
+	    ppstimes.real.tv_sec = (time_t)trunc(last_fixtime_real) + 1;
+	    ppstimes.real.tv_nsec = 0;  /* need to be fixed for 5Hz */
+	    ppstimes.clock = clock_ts;
 	    /*@-relaxtypes@*/
 
 	    /* check to see if we have a fresh timestamp from the
 	     * GPS serial input then use that */
-	    offset = (drift.real.tv_sec - drift.clock.tv_sec);
-	    offset += ((drift.real.tv_nsec - drift.clock.tv_nsec) / 1e9);
-	    delay = (drift.clock.tv_sec - last_fixtime_clock.tv_sec);
-	    delay += ((drift.clock.tv_nsec - last_fixtime_clock.tv_nsec) / 1e9);
-	    if (0.0 > delay || 1.0 < delay) {
+	    /*@-compdef@*/
+	    TS_SUB( &offset, &ppstimes.real, &ppstimes.clock);
+	    TS_SUB( &delay, &ppstimes.clock, &last_fixtime_clock);
+	    timespec_str( &delay, delay_str, sizeof(delay_str) );
+	    /*@+compdef@*/
+
+	    if ( 0> delay.tv_sec || 0 > delay.tv_nsec ) {
 		gpsd_report(&session->context->errout, LOG_RAW,
-			    "PPS: no current GPS seconds: %f\n",
-			    delay);
+			    "PPS: system clock went backwards: %.20s\n",
+			    delay_str);
+		log1 = "system clock went backwards";
+	    } else if ( ( 2 < delay.tv_sec) 
+	      || ( 1 == delay.tv_sec && 100000000 > delay.tv_nsec ) ) {
+                /* system clock could be slewing so allow 1.1 sec delay */
+		gpsd_report(&session->context->errout, LOG_RAW,
+			    "PPS: no current GPS seconds: %.20s\n",
+			    delay_str);
 		log1 = "timestamp out of range";
 	    } else {
 		/*@-compdef@*/
 		last_second_used = last_fixtime_real;
 		if (session->thread_report_hook != NULL) 
-		    log1 = session->thread_report_hook(session, &drift);
+		    log1 = session->thread_report_hook(session, &ppstimes);
 		else
 		    log1 = "no report hook";
 		if (session->context->pps_hook != NULL)
-		    session->context->pps_hook(session, &drift);
+		    session->context->pps_hook(session, &ppstimes);
 		/*@ -unrecog  (splint has no pthread declarations as yet) @*/
 		pthread_err = pthread_mutex_lock(&ppslast_mutex);
                 if ( 0 != pthread_err ) {
@@ -696,7 +709,7 @@ static /*@null@*/ void *gpsd_ppsmonitor(void *arg)
 		}
 		/*@ +unrecog @*/
 		/*@-type@*/ /* splint is confused about struct timespec */
-		session->ppslast = drift;
+		session->ppslast = ppstimes;
 		/*@+type@*/
 		session->ppscount++;
 		/*@ -unrecog (splint has no pthread declarations as yet) @*/
@@ -708,20 +721,23 @@ static /*@null@*/ void *gpsd_ppsmonitor(void *arg)
 		}
 		/*@ +unrecog @*/
 		/*@-type@*/ /* splint is confused about struct timespec */
-		timespec_str( &drift.clock, ts_str1, sizeof(ts_str1) );
-		timespec_str( &drift.real, ts_str2, sizeof(ts_str2) );
+		timespec_str( &ppstimes.clock, ts_str1, sizeof(ts_str1) );
+		timespec_str( &ppstimes.real, ts_str2, sizeof(ts_str2) );
 		gpsd_report(&session->context->errout, LOG_INF,
 			    "PPS hooks called with %.20s clock: %s real: %s\n",
 			    log1, ts_str1, ts_str2);
 		/*@+type@*/
 		/*@+compdef@*/
             }
+	    /*@-compdef@*/
 	    /*@-type@*/ /* splint is confused about struct timespec */
 	    timespec_str( &clock_ts, ts_str1, sizeof(ts_str1) );
+	    timespec_str( &offset, offset_str, sizeof(offset_str) );
 	    gpsd_report(&session->context->errout, LOG_PROG,
-		    "PPS edge %.20s @ %s offset %.9f\n",
-		    log1, ts_str1, offset);
+		    "PPS edge %.20s @ %s offset %.20s\n",
+		    log1, ts_str1, offset_str);
 	    /*@+type@*/
+	    /*@+compdef@*/
 	} else {
 	    gpsd_report(&session->context->errout, LOG_RAW,
 			"PPS edge rejected %.100s", log);
@@ -799,8 +815,8 @@ void pps_thread_stash_fixtime(struct gps_device_t *session,
     /*@ +unrecog @*/
 }
 
-int pps_thread_lastpps(struct gps_device_t *session, struct timedrift_t *td)
-/* return the drift at the time of the last PPS - only way we pass data out */
+int pps_thread_lastpps(struct gps_device_t *session, struct timedelta_t *td)
+/* return the delta at the time of the last PPS - only way we pass data out */
 {
     volatile int ret;
     /* pthread error return */
