@@ -9,7 +9,7 @@ GPS week+TOW, but this isn't possible in general.
 
 = Begin Sidebar: Why Leap Seconds =
 
-READ this carefully, and if there are errors, please correct.  An
+Read this carefully, and if there are errors, please correct.  An
 understanding of the following terms is critical to make sense of the
 situation, which would be farcical if it were not serious.
 
@@ -64,7 +64,7 @@ error (UT1 - UTC) build up. This is of the order of a few ms each midnight, not
 always the same way (think earthquakes that move the earth's crust).
 
 Once the error has built up substantially, every few years, we (and by
-"we", I mean M Daniel Gambis at the IERS) declare that a future
+"we", I mean M. Daniel Gambis at the IERS) declare that a future
 day will have 86401 secs.  This is the Leap Second.  Note that this
 often overcorrects, but if we wait a few months, the error will disappear.
 
@@ -79,16 +79,15 @@ Two last things:
     definitions above that causes leap seconds
  2. POSIX declares that there is no conflict, there are always 86400 SI
     secs in a day, and hence no leap seconds.  The fact that ostriches
-    survive in the wild indicates that this is not as mind-crushing
+    survive in the wild indicates that this is not as mind-crushingly
     wrong as it may seem.
 
 = End Sidebar =
 
-
 Date and time in GPS is represented as number of weeks mod 1024 from
-the start of zero second of 6 January 1980, and number of SI seconds into
-the week.  GPS time is not leap-second corrected, and has a constant
-offset from TAI, but not from UTC.
+1980-01-06T00:00.00Z, and number of SI seconds into the week.  GPS
+time is not leap-second corrected, and has a constant offset from TAI,
+but not from UTC.
 
 There are hence two issues with converting GPS Time to UTC:
 
@@ -104,18 +103,18 @@ the corrections when rollover occurs, as this may not be adequately
 tested.  We hence have to accept the UTC time reported by the device,
 while checking it on the sly).
 
-Satellites also broadcast a current leap-second
-correction which is updated on (theoretically) three-month boundaries
-according to rotational bulletins issued by the International
-Earth Rotation and Reference Systems Service (IERS).
-Historically all corrections have been made on six-month boundaries.
+Satellites also broadcast a current leap-second correction which is
+updated on (theoretically) three-month boundaries according to
+rotational bulletins issued by the International Earth Rotation and
+Reference Systems Service (IERS).  Historically all corrections have
+been made on six-month boundaries.
 
 The leap-second correction is only included in the satellite subframe
 broadcast, roughly once ever 20 minutes.  While the satellites do
 notify GPSes of upcoming leap-seconds, this notification is not
 necessarily processed correctly on consumer-grade devices, and will
 not be available at all when a GPS receiver has just
-cold-booted.  Thus, the time reported from NMEA devices, although
+cold-booted.  Thus, the time reported from GPS devices, although
 supposed to be UTC, may be offset by an integer number of seconds
 between a cold boot or leap second and the following
 subframe broadcast. 
@@ -128,24 +127,24 @@ divergence will normally be only one second or less.
 GPS date and time are subject to a rollover problem in the 10-bit week
 number counter, which will re-zero every 1024 weeks (roughly every 20
 years). The last rollover (and the first since GPS went live in 1980)
-was 0000 22 August 1999; the next would fall in 2019, but plans are
+was 1999-08-22T00:00:00Z; the next would fall in 2019, but plans are
 afoot to upgrade the satellite counters to 13 bits; this will delay
 the next rollover until 2173.
 
 For accurate time reporting, therefore, a GPS requires a supplemental
-time references sufficient to identify the current rollover period,
-e.g. accurate to within 512 weeks.  Many NMEA GPSes have a wired-in
+time reference sufficient to identify the current rollover period,
+e.g. accurate to within 512 weeks.  Many GPSes have a wired-in
 assumption about the UTC time of the last rollover and will thus report
 incorrect times outside the rollover period they were designed in.
 
 These conditions leave gpsd in a serious hole.  Actually there are several
 interrelated problems:
 
-1) Every NMEA device has some assumption about base epoch (date of
+1) Every device has some assumption about base epoch (date of
 last rollover) that we don't have access to.  Thus, there's no way to
 check whether a rollover the device wasn't prepared for has occurred
 before gpsd startup time (making the reported UTC date invalid)
-without some other time source.  (Some NMEA devices may keep a
+without some other time source.  (Some devices may keep a
 rollover count in NVRAM and avoid the problem; we can't tell when that's
 happening, either.)
 
@@ -168,10 +167,11 @@ is enabled, which requires 38400bps. Evermore GPSes can't be made to
 report it at all. Furthermore, before the almanac load the GPS may report
 a fixed (and possibly out of date) offset.
 
-Conclusion: if the system clock isn't accurate enough that we can deduce
-what rollover period we're in, we're utterly hosed. Furthermore, if it's
-not accurate to within a second and only NMEA devices are reporting,
-we don't even know what century it is!
+Conclusion: if the system clock isn't accurate enough that we can
+deduce what rollover period we're in, we're utterly
+hosed. Furthermore, if it's not accurate to within a second and only
+NMEA devices that don't emit ZDA are reporting, we don't even know
+what century it is!
 
 Therefore, we must assume the system clock is reliable to within a second.
 
@@ -216,14 +216,14 @@ void gpsd_time_init(struct gps_context_t *context, time_t starttime)
      * just before a century mark, but that case is probably doomed
      * anyhow because of 2-digit years.
      */
-    context->leap_seconds = LEAPSECOND_NOW;
-    context->century = CENTURY_BASE;
+    context->leap_seconds = BUILD_LEAPSECONDS;
+    context->century = BUILD_CENTURY;
     context->start_time = starttime;
 
     context->rollovers = (int)((context->start_time-GPS_EPOCH) / GPS_ROLLOVER);
 
     if (context->start_time < GPS_EPOCH)
-	gpsd_report(context->debug, LOG_ERROR,
+	gpsd_report(&context->errout, LOG_ERROR,
 		    "system time looks bogus, dates may not be reliable.\n");
     else {
 	/* we've forced the UTC timezone, so this is actually UTC */
@@ -236,7 +236,7 @@ void gpsd_time_init(struct gps_context_t *context, time_t starttime)
 	now->tm_year += 1900;
 	context->century = now->tm_year - (now->tm_year % 100);
 	(void)unix_to_iso8601((timestamp_t)context->start_time, scr, sizeof(scr));
-	gpsd_report(context->debug, LOG_INF,
+	gpsd_report(&context->errout, LOG_INF,
 		    "startup at %s (%d)\n",
 		    scr, (int)context->start_time);
     }
@@ -251,11 +251,11 @@ void gpsd_set_century(struct gps_device_t *session)
  */
 {
     char *end;
-    if (strstr((char *)session->packet.outbuffer, "Date:") != NULL) {
+    if (strstr((char *)session->lexer.outbuffer, "Date:") != NULL) {
 	int year;
-	unsigned char *cp = session->packet.outbuffer + 5;
+	unsigned char *cp = session->lexer.outbuffer + 5;
 	while (isspace(*cp))
-	    --cp;
+	    ++cp;
 	year = (int)strtol((char *)cp, &end, 10);
 	session->context->century = year - (year % 100);
     }
@@ -273,8 +273,8 @@ timestamp_t gpsd_utc_resolve(/*@in@*/struct gps_device_t *session)
      */
     timestamp_t t;
 
-    t = (timestamp_t)mkgmtime(&session->driver.nmea.date) +
-	session->driver.nmea.subseconds;
+    t = (timestamp_t)mkgmtime(&session->nmea.date) +
+	session->nmea.subseconds;
     session->context->valid &=~ GPS_TIME_VALID;
 
     /*
@@ -291,7 +291,7 @@ timestamp_t gpsd_utc_resolve(/*@in@*/struct gps_device_t *session)
     if (session->newdata.time < (timestamp_t)session->context->start_time) {
 	char scr[128];
 	(void)unix_to_iso8601(session->newdata.time, scr, sizeof(scr));
-	gpsd_report(session->context->debug, LOG_WARN,
+	gpsd_report(&session->context->errout, LOG_WARN,
 		    "GPS week rollover makes time %s (%f) invalid\n",
 		    scr, session->newdata.time);
     }
@@ -310,14 +310,14 @@ void gpsd_century_update(/*@in@*/struct gps_device_t *session, int century)
 	 * certainly it means that a century mark has passed while
 	 * gpsd was running, and we should trust the new ZDA year.
 	 */
-	gpsd_report(session->context->debug, LOG_WARN,
+	gpsd_report(&session->context->errout, LOG_WARN,
 		    "century rollover detected.\n");
 	session->context->century = century;
     } else if (session->context->start_time >= GPS_EPOCH && century < session->context->century) {
 	/*
 	 * This looks like a GPS week-counter rollover.
 	 */
-	gpsd_report(session->context->debug, LOG_WARN,
+	gpsd_report(&session->context->errout, LOG_WARN,
 		    "ZDA year less than clock year, "
 		    "probable GPS week rollover lossage\n");
 	session->context->valid &=~ CENTURY_VALID;
@@ -338,7 +338,7 @@ timestamp_t gpsd_gpstime_resolve(/*@in@*/struct gps_device_t *session,
      * to 13 bits.
      */
     if ((int)week < (session->context->gps_week & 0x3ff)) {
-	gpsd_report(session->context->debug, LOG_INF,
+	gpsd_report(&session->context->errout, LOG_INF,
 		    "GPS week 10-bit rollover detected.\n");
 	++session->context->rollovers;
     }
